# -*- coding: utf-8 -*-
#
# Copyright (C) 2011-2012 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Minesweeper game for WeeChat.
# (mouse supported with WeeChat >= 0.3.6)
#
# History:
#
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.6: make script compatible with Python 3.x
# 2011-10-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.5: stop timer when game ends (win with flags remaining) or when
#                  minesweeper buffer is not displayed
# 2011-10-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.4: fix end of game when player blows up
# 2011-10-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.3: end game (win) if all squares without mines are explored
# 2011-10-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.2: add option "utf8" (to disable utf-8 chars for grid and flags)
# 2011-10-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
#

SCRIPT_NAME    = 'minesweeper'
SCRIPT_AUTHOR  = 'Sebastien Helleu <flashcode@flashtux.org>'
SCRIPT_VERSION = '0.6'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC    = 'Minesweeper game'

SCRIPT_COMMAND = 'minesweeper'

import_ok = True

try:
    import weechat
except ImportError:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: http://www.weechat.org/')
    import_ok = False

try:
    import random, copy
except ImportError as message:
    print('Missing package(s) for %s: %s' % (SCRIPT_NAME, message))
    import_ok = False

minesweeper = {
    'buffer'      : '',
    'board'       : [],
    'mines'       : { 8: 10, 15: 35, 30: 100, 50: 350 },
    'size'        : 8,
    'zoom'        : 1,
    'x'           : 7,
    'y'           : 7,
    'flags'       : 0,
    'timer'       : '',
    'time'        : 0,
    'end'         : '',
    'endmsg'      : { 'win': ('lightgreen', 'Congratulations!'), 'lose': ('lightred', 'You blew up!') },
    'color_digits': [],
    'cursor'      : False,
    'cheat'       : False,
}

# script options
minesweeper_settings_default = {
    'color_grid'        : ('243',     'default',   'color for grid'),
    'color_square_bg'   : ('239',     'blue',      'color for square (background)'),
    'color_flag'        : ('white',   'yellow',    'color for flag'),
    'color_cursor_bg'   : ('19',      'lightblue', 'color for cursor (background)'),
    'color_mine'        : ('white',   'white',     'color for mine'),
    'color_mine_bg'     : ('97',      'magenta',   'color for mine (background)'),
    'color_explosion_bg': ('172',     'red',       'color for explosion (background)'),
    'color_digits'      : ('21,126,34,209,201,185,15,160,9',
                           'blue,magenta,green,brown,lightmagenta,default,white,red,lightred',
                           'comma-separated list of 9 colors (for digits 1-9)'),
    'utf8'              : ('on',      'on',        'use utf-8 chars to draw grid and flags (your terminal/font must support these chars)'),
    'zoom'              : ('',        '',          'zoom for board: 0 or 1 (size of squares: 0 = 4x2, 1 = 6x3 (better), empty means automatic zoom according to size of window)'),
}
minesweeper_settings = {}

# mouse keys
minesweeper_mouse_keys = { '@chat(python.minesweeper):button1*': '/window ${_window_number};hsignal:minesweeper_mouse',
                           '@chat(python.minesweeper):button2*': '/window ${_window_number};hsignal:minesweeper_mouse' }

def minesweeper_display_status():
    """Display status line below board."""
    global minesweeper
    if not minesweeper['buffer']:
        return
    msgend = ''
    if minesweeper['end']:
        msgend = '%s%s' % (weechat.color(minesweeper['endmsg'][minesweeper['end']][0]), minesweeper['endmsg'][minesweeper['end']][1])
    else:
        if minesweeper['flags'] == 0:
            msgend = '%sSome bad flags, remove and go on!' % weechat.color('yellow')
    hours = minesweeper['time'] // 3600
    minutes = (minesweeper['time'] % 3600) // 60
    seconds = (minesweeper['time'] % 3600) % 60
    if minesweeper_settings['utf8'] == 'on':
        flag = '⚑'
    else:
        flag = 'p'
    weechat.prnt_y(minesweeper['buffer'], 2 + (minesweeper['size'] * (minesweeper['zoom'] + 2)),
                   '%s %3d%s/%-3d%s%5d:%02d:%02d  %s' % (flag, minesweeper['flags'], weechat.color('green'),
                                                         minesweeper['mines'][minesweeper['size']],
                                                         weechat.color('reset'), hours, minutes, seconds,
                                                         msgend))

def minesweeper_display(clear=False):
    """Display status and board."""
    global minesweeper, minesweeper_settings
    if not minesweeper['buffer']:
        return
    if clear:
        weechat.buffer_clear(minesweeper['buffer'])
    if minesweeper_settings['utf8'] == 'on':
        hbar = '▁'
        vbar = '▕'
        flag = '⚑'
    else:
        hbar = '_'
        vbar = '|'
        flag = 'p'
    str_grid = '%s%s%s%s ' % (hbar, hbar, hbar, hbar * minesweeper['zoom'] * 2)
    weechat.prnt_y(minesweeper['buffer'], 0, '%s%s' % (weechat.color(minesweeper_settings['color_grid']), str_grid * minesweeper['size']))
    color_explosion = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_explosion_bg'])
    color_explosion_text = '%s,%s' % (minesweeper_settings['color_flag'], minesweeper_settings['color_explosion_bg'])
    color_mine = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_mine_bg'])
    color_mine_text = '%s,%s' % (minesweeper_settings['color_mine'], minesweeper_settings['color_mine_bg'])
    for y, line in enumerate(minesweeper['board']):
        if minesweeper['zoom'] == 0:
            str_lines = ['', '']
        else:
            str_lines = ['', '', '']
        for x, status in enumerate(line):
            if minesweeper['cursor'] and minesweeper['x'] == x and minesweeper['y'] == y:
                color_nostatus = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_cursor_bg'])
                color_flag = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_cursor_bg'])
                color_flag_text = '%s,%s' % (minesweeper_settings['color_flag'], minesweeper_settings['color_cursor_bg'])
                color_digit = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_cursor_bg'])
                color_digit_text_bg = ',%s' % minesweeper_settings['color_cursor_bg']
            else:
                color_nostatus = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_square_bg'])
                color_flag = '%s,%s' % (minesweeper_settings['color_grid'], minesweeper_settings['color_square_bg'])
                color_flag_text = '%s,%s' % (minesweeper_settings['color_flag'], minesweeper_settings['color_square_bg'])
                color_digit = '%s,default' % minesweeper_settings['color_grid']
                color_digit_text_bg = ',default'
            if status[1] == ' ':
                char = ' '
                if status[0] and minesweeper['cheat']:
                    char = '*'
                if minesweeper['zoom'] == 0:
                    str_lines[0] += '%s %s %s%s' % (weechat.color(color_nostatus), char, vbar, weechat.color('reset'))
                    str_lines[1] += '%s%s%s%s' % (weechat.color(color_nostatus), hbar * 3, vbar, weechat.color('reset'))
                else:
                    str_lines[0] += '%s     %s%s' % (weechat.color(color_nostatus), vbar, weechat.color('reset'))
                    str_lines[1] += '%s  %s  %s%s' % (weechat.color(color_nostatus), char, vbar, weechat.color('reset'))
                    str_lines[2] += '%s%s%s%s' % (weechat.color(color_nostatus), hbar * 5, vbar, weechat.color('reset'))
            elif status[1] == 'F':
                if minesweeper['zoom'] == 0:
                    str_lines[0] += '%s %s%s%s %s%s' % (weechat.color(color_flag), weechat.color(color_flag_text), flag, weechat.color(color_flag), vbar, weechat.color('reset'))
                    str_lines[1] += '%s%s%s%s' % (weechat.color(color_flag), hbar * 3, vbar, weechat.color('reset'))
                else:
                    str_lines[0] += '%s     %s%s' % (weechat.color(color_flag), vbar, weechat.color('reset'))
                    str_lines[1] += '%s  %s%s%s  %s%s' % (weechat.color(color_flag), weechat.color(color_flag_text), flag, weechat.color(color_flag), vbar, weechat.color('reset'))
                    str_lines[2] += '%s%s%s%s' % (weechat.color(color_flag), hbar * 5, vbar, weechat.color('reset'))
            elif status[1].isdigit():
                char = status[1]
                if char == '0':
                    char = ' '
                    color_digit_text = 'default'
                else:
                    color_digit_text = minesweeper['color_digits'][int(status[1]) - 1]
                if minesweeper['zoom'] == 0:
                    str_lines[0] += '%s %s%s%s %s%s' % (
                        weechat.color(color_digit), weechat.color(color_digit_text + color_digit_text_bg), char, weechat.color(color_digit), vbar, weechat.color('reset'))
                    str_lines[1] += '%s%s%s%s' % (weechat.color(color_digit), hbar * 3, vbar, weechat.color('reset'))
                else:
                    str_lines[0] += '%s     %s%s' % (weechat.color(color_digit), vbar, weechat.color('reset'))
                    str_lines[1] += '%s  %s%s%s  %s%s' % (
                        weechat.color(color_digit), weechat.color(color_digit_text + color_digit_text_bg), char, weechat.color(color_digit), vbar, weechat.color('reset'))
                    str_lines[2] += '%s%s%s%s' % (weechat.color(color_digit), hbar * 5, vbar, weechat.color('reset'))
            elif status[1] == '+':
                if minesweeper['zoom'] == 0:
                    str_lines[0] += '%s %s*%s %s%s' % (weechat.color(color_mine), weechat.color(color_mine_text), weechat.color(color_mine), vbar, weechat.color('reset'))
                    str_lines[1] += '%s%s%s%s' % (weechat.color(color_mine), hbar * 3, vbar, weechat.color('reset'))
                else:
                    str_lines[0] += '%s     %s%s' % (weechat.color(color_mine), vbar, weechat.color('reset'))
                    str_lines[1] += '%s  %s*%s  %s%s' % (weechat.color(color_mine), weechat.color(color_mine_text), weechat.color(color_mine), vbar, weechat.color('reset'))
                    str_lines[2] += '%s%s%s%s' % (weechat.color(color_mine), hbar * 5, vbar, weechat.color('reset'))
            elif status[1] == '*':
                if minesweeper['zoom'] == 0:
                    str_lines[0] += '%s %s*%s %s%s' % (
                        weechat.color(color_explosion), weechat.color(color_explosion_text), weechat.color(color_explosion), vbar, weechat.color('reset'))
                    str_lines[1] += '%s%s%s%s' % (weechat.color(color_explosion), hbar * 3, vbar, weechat.color('reset'))
                else:
                    str_lines[0] += '%s     %s%s' % (weechat.color(color_explosion), vbar, weechat.color('reset'))
                    str_lines[1] += '%s  %s*%s  %s%s' % (
                        weechat.color(color_explosion), weechat.color(color_explosion_text), weechat.color(color_explosion), vbar, weechat.color('reset'))
                    str_lines[2] += '%s%s%s%s' % (weechat.color(color_explosion), hbar * 5, vbar, weechat.color('reset'))
        for i, str_line in enumerate(str_lines):
            weechat.prnt_y(minesweeper['buffer'], 1 + (y * len(str_lines)) + i, str_line)
    minesweeper_display_status()

def minesweeper_adjust_zoom():
    """Choose zoom according to size of window."""
    global minesweeper, minesweeper_settings
    minesweeper['zoom'] = -1
    if minesweeper_settings['zoom']:
        try:
            minesweeper['zoom'] = int(minesweeper_settings['zoom'])
        except:
            minesweeper['zoom'] = -1
        if minesweeper['zoom'] > 1:
            minesweeper['zoom'] = 1
    if minesweeper['zoom'] < 0:
        width = weechat.window_get_integer(weechat.current_window(), 'win_chat_width')
        height = weechat.window_get_integer(weechat.current_window(), 'win_chat_height')
        minesweeper['zoom'] = 0
        if width >= minesweeper['size'] * 6 and height >= 1 + (minesweeper['size'] * 3) + 2:
            minesweeper['zoom'] = 1
    if minesweeper['zoom'] < 0:
        minesweeper['zoom'] = 0

def minesweeper_set_colors():
    """Set list of colors using settings."""
    global minesweeper, minesweeper_settings, minesweeper_settings_default
    minesweeper['color_digits'] = minesweeper_settings['color_digits'].split(',')
    if len(minesweeper['color_digits']) != 9:
        weechat.prnt('', '%sminesweeper: invalid colors (list must have 9 colors)' % weechat.prefix('error'))
        minesweeper['color_digits'] = minesweeper_settings_default['color_digits'][0].split(',')

def minesweeper_config_cb(data, option, value):
    """Called when a script option is changed."""
    global minesweeper_settings
    pos = option.rfind('.')
    if pos > 0:
        name = option[pos+1:]
        if name in minesweeper_settings:
            minesweeper_settings[name] = value
            if name == 'color_digits':
                minesweeper_set_colors()
            elif name == 'zoom':
                minesweeper_adjust_zoom()
    minesweeper_display()
    return weechat.WEECHAT_RC_OK

def minesweeper_timer_cb(data, remaining_calls):
    """Callback for timer."""
    global minesweeper
    if minesweeper['buffer'] and weechat.buffer_get_integer(minesweeper['buffer'], 'num_displayed') > 0:
        minesweeper['time'] += 1
    minesweeper_display_status()
    return weechat.WEECHAT_RC_OK

def minesweeper_timer_start():
    """Start timer."""
    global minesweeper
    if not minesweeper['timer']:
        minesweeper['time'] = 0
        minesweeper['timer'] = weechat.hook_timer(1000, 0, 0, 'minesweeper_timer_cb', '')

def minesweeper_timer_stop():
    """Stop timer."""
    global minesweeper
    if minesweeper['timer']:
        weechat.unhook(minesweeper['timer'])
        minesweeper['timer'] = ''

def minesweeper_new_game():
    """Create a new game: initialize board and some variables."""
    global minesweeper
    minesweeper['board'] = []
    for y in range(0, minesweeper['size']):
        line = []
        for x in range(0, minesweeper['size']):
            line.append([False, ' '])
        minesweeper['board'].append(line)
    for i in range(0, minesweeper['mines'][minesweeper['size']]):
        while 1:
            x = random.randint(0, minesweeper['size'] - 1)
            y = random.randint(0, minesweeper['size'] - 1)
            if not minesweeper['board'][y][x][0]:
                break
        minesweeper['board'][y][x][0] = True
    minesweeper['x'] = minesweeper['size'] // 2
    minesweeper['y'] = minesweeper['size'] // 2
    minesweeper['flags'] = minesweeper['mines'][minesweeper['size']]
    minesweeper_timer_stop()
    minesweeper['time'] = 0
    minesweeper['end'] = ''
    minesweeper_display(clear=True)

def minesweeper_end(end):
    """End of game."""
    global minesweeper
    minesweeper['end'] = end
    minesweeper_timer_stop()

def minesweeper_change_size(add):
    """Change size of board."""
    global minesweeper
    keys = sorted(minesweeper['mines'])
    index = keys.index(minesweeper['size']) + add
    if index >= 0 and index < len(keys):
        minesweeper['size'] = keys[index]
        minesweeper['count_max'] = minesweeper['mines'][minesweeper['size']]
        weechat.buffer_clear(minesweeper['buffer'])
        minesweeper_adjust_zoom()
        minesweeper_new_game()

def minesweeper_input_buffer(data, buffer, input):
    """Input data in minesweeper buffer."""
    global minesweeper
    if input:
        args = input.split(' ')
        if args[0] in ('n', 'new'):
            minesweeper_new_game()
        elif args[0] in ('q', 'quit'):
            weechat.buffer_close(minesweeper['buffer'])
        elif args[0] == '+':
            minesweeper_change_size(+1)
        elif args[0] == '-':
            minesweeper_change_size(-1)
    return weechat.WEECHAT_RC_OK

def minesweeper_close_buffer(data, buffer):
    """Called when minesweeper buffer is closed."""
    global minesweeper
    minesweeper_timer_stop()
    minesweeper['buffer'] = ''
    return weechat.WEECHAT_RC_OK

def minesweeper_init():
    """Init minesweeper: create buffer, adjust zoom, new game."""
    global minesweeper
    if minesweeper['buffer']:
        return
    minesweeper['buffer'] = weechat.buffer_search('python', 'minesweeper')
    if not minesweeper['buffer']:
        minesweeper['buffer'] = weechat.buffer_new('minesweeper', 'minesweeper_input_buffer', '', 'minesweeper_close_buffer', '')
        if minesweeper['buffer']:
            weechat.buffer_set(minesweeper['buffer'], 'type', 'free')
            weechat.buffer_set(minesweeper['buffer'], 'title',
                               'Minesweeper! | alt-space or mouse-b1: explore, alt-f or mouse-b2: flag, alt-n: new game, '
                               'alt-+/-: adjust board zoom | '
                               'Command line: (n)ew, +/-: change size, (q)uit')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta2-A', '/minesweeper up')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta2-B', '/minesweeper down')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta2-D', '/minesweeper left')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta2-C', '/minesweeper right')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta-f',  '/minesweeper flag')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta- ',  '/minesweeper explore')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta-n',  '/minesweeper new')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta-+',  '/minesweeper zoom')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta--',  '/minesweeper dezoom')
            weechat.buffer_set(minesweeper['buffer'], 'key_bind_meta-c',  '/minesweeper cheat')
    if minesweeper['buffer']:
        minesweeper_adjust_zoom()
        minesweeper_new_game()

def minesweeper_number_around(x, y):
    """Return number of mines around."""
    global minesweeper
    around = 0
    if x >= 0 and x <= minesweeper['size'] - 1 and y >= 0 and y <= minesweeper['size'] - 1:
        for diff in ((-1, -1), (0, -1), (1, -1), (-1, 0), (1, 0), (-1, 1), (0, 1), (1, 1)):
            new_x = x + diff[0]
            new_y = y + diff[1]
            if new_x >= 0 and new_x <= minesweeper['size'] - 1 and new_y >= 0 and new_y <= minesweeper['size'] - 1:
                if minesweeper['board'][new_y][new_x][0]:
                    around += 1
    return around

def minesweeper_show_solution():
    """Show solution when game has ended."""
    global minesweeper
    for y, line in enumerate(minesweeper['board']):
        for x, status in enumerate(line):
            if status[1] != '*':
                if status[0]:
                    status[1] = '+'
                else:
                    number_around = minesweeper_number_around(x, y)
                    status[1] = '%d' % number_around

def minesweeper_all_flags_ok():
    global minesweeper
    number_ok = 0
    for x, line in enumerate(minesweeper['board']):
        for y, status in enumerate(line):
            if status[0] and status[1] == 'F':
                number_ok += 1
    return number_ok == minesweeper['mines'][minesweeper['size']]

def minesweeper_all_squares_explored():
    global minesweeper
    explored = 0
    for x, line in enumerate(minesweeper['board']):
        for y, status in enumerate(line):
            if status[1] == '*':
                return False
            if not status[0] and status[1].isdigit():
                explored += 1
    return explored == (minesweeper['size'] * minesweeper['size']) - minesweeper['mines'][minesweeper['size']]

def minesweeper_flag(x, y):
    """Add/remove flag."""
    global minesweeper
    status = minesweeper['board'][y][x]
    if status[1] in ' F':
        if status[1] == ' ':
            if minesweeper['flags'] == 0:
                return
            status[1] = 'F'
            minesweeper['flags'] -= 1
            if minesweeper['flags'] == 0 and minesweeper_all_flags_ok():
                minesweeper_end('win')
        else:
            status[1] = ' '
            minesweeper['flags'] += 1
        minesweeper_display()

def minesweeper_explore(x, y):
    """Explore!"""
    global minesweeper
    if not minesweeper['end'] and x >= 0 and x <= minesweeper['size'] - 1 and y >= 0 and y <= minesweeper['size'] - 1:
        status = minesweeper['board'][y][x]
        if status[1] == ' ':
            if status[0]:
                status[1] = '*'
                minesweeper_end('lose')
                minesweeper_show_solution()
            else:
                number_around = minesweeper_number_around(x, y)
                status[1] = '%d' % number_around
                if number_around == 0:
                    for diff in ((-1, -1), (0, -1), (1, -1), (-1, 0), (1, 0), (-1, 1), (0, 1), (1, 1)):
                        minesweeper_explore(x + diff[0], y + diff[1])

def minesweeper_cmd_cb(data, buffer, args):
    """The /minesweeper command."""
    global minesweeper
    if args in ('16col', '256col'):
        index = 0
        if args == '16col':
            index = 1
        for option, value in minesweeper_settings_default.items():
            if option.startswith('color_'):
                weechat.config_set_plugin(option, value[index])
                minesweeper_settings[option] = value[index]
        minesweeper_set_colors()
        minesweeper_display()
        return weechat.WEECHAT_RC_OK
    minesweeper_init()
    if minesweeper['buffer']:
        weechat.buffer_set(minesweeper['buffer'], 'display', '1')
    if not minesweeper['end']:
        if args == 'up':
            minesweeper['cursor'] = True
            if minesweeper['y'] > 0:
                minesweeper['y'] -= 1
                minesweeper_display()
        elif args == 'down':
            minesweeper['cursor'] = True
            if minesweeper['y'] < minesweeper['size'] - 1:
                minesweeper['y'] += 1
                minesweeper_display()
        elif args == 'left':
            minesweeper['cursor'] = True
            if minesweeper['x'] > 0:
                minesweeper['x'] -= 1
                minesweeper_display()
        elif args == 'right':
            minesweeper['cursor'] = True
            if minesweeper['x'] < minesweeper['size'] - 1:
                minesweeper['x'] += 1
                minesweeper_display()
        elif args == 'flag':
            minesweeper_timer_start()
            minesweeper['cursor'] = True
            minesweeper_flag(minesweeper['x'], minesweeper['y'])
        elif args == 'explore':
            minesweeper_timer_start()
            minesweeper['cursor'] = True
            minesweeper_explore(minesweeper['x'], minesweeper['y'])
            if not minesweeper['end'] and minesweeper_all_squares_explored():
                minesweeper_end('win')
            minesweeper_display()
    if args == 'new':
        minesweeper_new_game()
    elif args == 'zoom':
        if minesweeper['zoom'] < 1:
            minesweeper['zoom'] += 1
            minesweeper_display(True)
    elif args == 'dezoom':
        if minesweeper['zoom'] > 0:
            minesweeper['zoom'] -= 1
            minesweeper_display(True)
    elif args == 'cheat':
        minesweeper['cheat'] = not minesweeper['cheat']
        minesweeper_display()
    return weechat.WEECHAT_RC_OK

def minesweeper_mouse_cb(data, hsignal, hashtable):
    """Mouse callback."""
    global minesweeper
    if minesweeper['end']:
        minesweeper_new_game()
    else:
        minesweeper['cursor'] = False
        x = int(hashtable.get('_chat_line_x', '-1'))
        y = int(hashtable.get('_chat_line_y', '-1'))
        key = hashtable.get('_key', '')
        if x >= 0 and y >= 1:
            x = x // (4 + (minesweeper['zoom'] * 2))
            y = (y - 1) // (minesweeper['zoom'] + 2)
            if x >= 0 and x <= minesweeper['size'] - 1 and y >= 0 and y <= minesweeper['size'] - 1:
                minesweeper_timer_start()
                if key.startswith('button1'):
                    minesweeper_explore(x, y)
                    if not minesweeper['end'] and minesweeper_all_squares_explored():
                        minesweeper_end('win')
                    minesweeper_display()
                elif key.startswith('button2'):
                    minesweeper_flag(x, y)
    return weechat.WEECHAT_RC_OK

if __name__ == '__main__' and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, '', ''):
        # set default settings
        version = weechat.info_get('version_number', '') or 0
        for option, value in minesweeper_settings_default.items():
            if weechat.config_is_set_plugin(option):
                minesweeper_settings[option] = weechat.config_get_plugin(option)
            else:
                weechat.config_set_plugin(option, value[0])
                minesweeper_settings[option] = value[0]
            if int(version) >= 0x00030500:
                weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[2], value[0]))
        minesweeper_set_colors()

        # mouse support
        if int(version) >= 0x00030600:
            weechat.key_bind('mouse', minesweeper_mouse_keys)
            weechat.hook_hsignal('minesweeper_mouse', 'minesweeper_mouse_cb', '')

        # detect config changes
        weechat.hook_config('plugins.var.python.%s.*' % SCRIPT_NAME, 'minesweeper_config_cb', '')

        # add command
        weechat.hook_command(SCRIPT_COMMAND, 'Minesweeper game.', '[16col|256col]',
                             '16col: set colors using basic colors (if your terminal or your WeeChat does not support 256 colors)\n'
                             '256col: set colors using 256 colors mode (default)\n\n'
                             '256 colors mode is highly recommended (WeeChat >= 0.3.5).\n'
                             'Mouse is recommended (WeeChat >= 0.3.6).\n\n'
                             'Instructions:\n'
                             '- use mouse left button (or alt-space) to explore a square, if you think there is no mine under '
                             '(if there is a mine, you lose!)\n'
                             '- use mouse right button (or alt-f) to put/remove flag on square, if you think there is a mine under\n'
                             '- you win if you put all flags on mines, of if all squares without mine are explored.\n\n'
                             'Good luck!',
                             '16col|256col', 'minesweeper_cmd_cb', '')

        # if buffer already exists (after /upgrade), init minesweeper
        if weechat.buffer_search('python', 'minesweeper'):
            minesweeper_init()



# -*- coding: utf-8 -*-
#
# Copyright 2016 Gerard Ryan <gerard@ryan.lt>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Changelog:
#
# 2016-11-05: Gerard Ryan <gerard@ryan.lt>
#     0.1.0 : Initial version
#
# 2016-12-17: Gerard Ryan <gerard@ryan.lt>
#     0.2.0 : - Prevent changing manually-set away status
#             - Allow configuration of message and poll time
#             - Tolerate gnome-shell crashes
#
# 2021-04-05: Sébastien Helleu <flashcode@flashtux.org>
#     0.2.1 : - Add missing call to infolist_free
#
# Contributions welcome at:
# https://github.com/grdryn/weechat-gnome-screensaver-away

from __future__ import print_function

try:
    import dbus
    import weechat
except ImportError:
    print('This program is intended to be run by weechat, and expects the')
    print('dbus-python library to be installed.')

SCRIPT_NAME    = 'gnome-screensaver-away'
SCRIPT_AUTHOR  = 'Gerard Ryan <gerard@ryan.lt>'
SCRIPT_VERSION = '0.2.1'
SCRIPT_LICENSE = 'GPLv3+'
SCRIPT_DESC    = 'Set away status based on GNOME ScreenSaver status'

def set_default_configuration(away_msg, poll_interval):
    if not weechat.config_get_plugin('away_msg'):
        weechat.config_set_plugin('away_msg', away_msg)
    if not weechat.config_get_plugin('poll_interval'):
        weechat.config_set_plugin('poll_interval', str(poll_interval))

def get_poll_interval_safely():
    poll_interval = 5000 # default

    try:
        poll_interval = int(weechat.config_get_plugin('poll_interval'))
    except ValueError:
        weechat.prnt('', 'poll_interval is not an int, falling back to default')

    return poll_interval

# TODO: Try to track the away status of each IRC server independently?
def check_away_status():
    away = (False, False)
    irc_servers = weechat.infolist_get("irc_server", "", "")

    while weechat.infolist_next(irc_servers):
        auto_away_msg    = weechat.config_get_plugin('away_msg')
        current_away_msg = weechat.infolist_string(irc_servers, "away_message")

        is_away_by_me = current_away_msg == auto_away_msg
        is_away       = bool(weechat.infolist_integer(irc_servers, "is_away"))
        away          = (is_away, is_away_by_me)

    weechat.infolist_free(irc_servers)

    return away

def check_screensaver_status(data, remaining_calls):
    away, auto_away = check_away_status()

    screensaver = dbus.SessionBus().get_object(
        'org.gnome.ScreenSaver', '/org/gnome/ScreenSaver')
    screensaver_on = screensaver.GetActive(
        dbus_interface='org.gnome.ScreenSaver')

    if screensaver_on and not away:
        weechat.command('', '/away -all {0}'.format(
            weechat.config_get_plugin('away_msg')))
    elif away and auto_away and not screensaver_on:
        weechat.command('', '/away -all')

    return weechat.WEECHAT_RC_OK

if __name__ == '__main__':
    weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                 SCRIPT_DESC, '', '')

    set_default_configuration('I am away', 5000)

    poll_interval = get_poll_interval_safely()
    weechat.hook_timer(poll_interval, 0, 0, 'check_screensaver_status', '')



# pagetitle plugin for weechat-0.3.0
#
#  /pagetitle http://tech.slashdot.org/tech/08/11/12/199215.shtml
#  <user> http://tech.slashdot.org/tech/08/11/12/199215.shtml
#  ('Slashdot | Microsoft's "Dead Cow" Patch Was 7 Years In the Making')
#
# xororand @ irc://irc.freenode.net/#weechat
#
# 2021-06-05, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.6: make script compatible with Python 3,
#                  rename command /pt to /pagetitle, fix PEP8 errors
# 2009-05-02, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.5: sync with last API changes

from html import unescape
from urllib.error import URLError
from urllib.request import Request, urlopen

import re
import weechat

MAX_TITLE_LENGTH = 100

regex_url = re.compile("""https?://[^ ]+""")


def get_page_title(url):
    """Retrieve the HTML <title> from a webpage."""
    req = Request(
        url,
        headers={
            "User-agent": "Mozilla/5.0 (weechat/pagetitle)",
        },
    )
    try:
        head = urlopen(req).read(8192).decode("utf-8", errors="ignore")
    except URLError:
        return ""
    match = re.search("(?i)<title>(.*?)</title>", head)
    return unescape(match.group(1)) if match else ""


def add_page_titles(data):
    """Add page titles for all URLs of a message."""
    buffer, msg = data.split(";", 1)

    def url_replace(match):
        url = match.group()
        title = get_page_title(url)
        if len(title) > MAX_TITLE_LENGTH:
            title = "%s [...]" % title[0:MAX_TITLE_LENGTH]
        url = "%s ('%s')" % (url, title)
        return url

    msg = regex_url.sub(url_replace, msg)
    return f"{buffer};{msg}"


def process_cb(data, command, rc, stdout, stderr):
    """Process callback."""
    buffer, msg = stdout.split(";", 1)
    weechat.command(buffer, "/say %s" % msg)
    return weechat.WEECHAT_RC_OK


# /pagetitle http://foo
def cmd_pagetitle_cb(data, buffer, args):
    if len(args) == 0:
        return weechat.WEECHAT_RC_ERROR
    weechat.hook_process(
        "func:add_page_titles",
        30 * 1000,
        "process_cb",
        f"{buffer};{args}",
    )
    return weechat.WEECHAT_RC_OK


weechat.register(
    "pagetitle",
    "xororand",
    "0.6",
    "GPL3",
    """Adds HTML titles to http:// urls in your message.""",
    "",
    "",
)
desc = """\
Sends a message to the current buffer and adds HTML titles to http:// URLs.
Example: /pagetitle check this out: http://xkcd.com/364/
<you> check this out: http://xkcd.com/364/ (xkcd - A webcomic of romance, \
sarcasm, math and language)"""
weechat.hook_command(
    "pagetitle",
    desc,
    "message",
    "message with URL(s)",
    "",
    "cmd_pagetitle_cb",
    "",
)

# vim:set ts=4 sw=4 noexpandtab nowrap foldmethod=marker:



# -*- coding: utf-8 -*-
# (this script requires WeeChat 0.3.0 or newer)
#
# Copyright 2018 tx <trqx@goat.si>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
# self-censor stuff you were going to say
#
# configuration examples:
#
# set comma separated words:
# /set plugins.var.python.selfcensor.censors "shit,fuck"
# /set plugins.var.python.selfcensor.censors "://reddit.com,://youtube.com"
# /set plugins.var.python.selfcensor.censors ""
#
# set warning message:
# /set plugins.var.python.selfcensor.warning "NOPE!"
#
# set tourette message (automatically sent instead of your message):
# /set plugins.var.python.selfcensor.tourette "FUCK"
#
# disable tourette:
# /set plugins.var.python.selfcensor.tourette ""
#
# History:
#
# 2018-09-20, tx <trqx@goat.si>
#    v0.1: initial release

import weechat as w
import re

SCRIPT_NAME = "selfcensor"
SCRIPT_AUTHOR = "tx <trqx@goat.si"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "MIT"
SCRIPT_DESC = "self-censor stuff you were going to say"

# script options
settings = {"censors": "", "warning": "🔥", "tourette": ""}


if w.register(
    SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""
):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)

    # Hooks we want to hook
    hook_command_run = {"input": ("/input return", "command_run_input")}
    # Hook all hooks !
    for hook, value in hook_command_run.items():
        w.hook_command_run(value[0], value[1], "")


def command_run_input(data, buffer, command):
    """ Function called when a command "/input xxxx" is run """
    if command == "/input return":  # As in enter was pressed.

        # Get input contents
        input_s = w.buffer_get_string(buffer, "input")

        # Skip modification of settings
        if input_s.startswith("/set "):
            return w.WEECHAT_RC_OK

        # Iterate censored stuff
        for censor in w.config_get_plugin("censors").split(","):
            if censor:
                if censor in input_s:
                    warning = "{} ({})".format(w.config_get_plugin("warning"), censor)
                    input_s = w.config_get_plugin("tourette")
                    w.command("", "/print " + warning)
                    break

        # Spit it out
        w.buffer_set(buffer, "input", input_s)
    return w.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2020-06-05, Normen Hansen <normen667@gmail.com>
#   version 0.7: add option to change prefix of entered text
# 2019-06-17, Brad Hubbard <bhubbard@redhat.com>
#   version 0.6: replace iteritems with items for python3 compatability
# 2011-07-17, Sébastien Helleu <flashcode@flashtux.org>
#   version 0.5: allow empty value for pairs or words
# 2011-02-01, xt
#   version 0.4: improve regexp for word replacement
# 2010-11-26, xt <xt@bash.no>
#   version 0.3: don't replace in /set commands
# 2009-10-27, xt <xt@bash.no>
#   version 0.2: also replace on words
# 2009-10-22, xt <xt@bash.no>
#   version 0.1: initial release

import weechat as w
import re

SCRIPT_NAME    = "text_replace"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.7"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Replaces text you write with replacement text"

# script options
settings = {
        'replacement_pairs': '(:=:),):=:(',   # pairs separated by , orig text and replacement separated by =
        'replacement_words': 'hhe=heh',       # words separated by , orig text and replacement separated by =
        'replacement_prefixes': ':=/',       # strings separated by , orig prefix and replacement separated by =
}



if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)

    # Hooks we want to hook
    hook_command_run = {
        "input" : ("/input return",  "command_run_input"),
    }
    # Hook all hooks !
    for hook, value in hook_command_run.items():
        w.hook_command_run(value[0], value[1], "")


def command_run_input(data, buffer, command):
    """ Function called when a command "/input xxxx" is run """
    if command == "/input return": # As in enter was pressed.

        # Get input contents
        input_s = w.buffer_get_string(buffer, 'input')

        # Skip modification of settings
        if input_s.startswith('/set '):
            return w.WEECHAT_RC_OK

        # Iterate transformation pairs
        for replace_item in w.config_get_plugin('replacement_pairs').split(','):
            if replace_item:
                orig, replaced = replace_item.split('=')
                input_s = input_s.replace(orig, replaced)
        # Iterate words
        for replace_item in w.config_get_plugin('replacement_words').split(','):
            if replace_item:
                orig, replaced = replace_item.split('=')
                # Search for whitespace+word+whitespace and replace the word
                input_s = re.sub('(\s+|^)%s(\s+|$)' %orig, '\\1%s\\2' %replaced, input_s)
        # Iterate prefixes
        for replace_item in w.config_get_plugin('replacement_prefixes').split(','):
            if replace_item:
                orig, replaced = replace_item.split('=')
                if input_s.startswith(orig):
                    input_s = input_s.replace(orig, replaced, 1)

        # Spit it out
        w.buffer_set(buffer, 'input', input_s)
    return w.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
# Author: Josh Dick <josh@joshdick.net>
# <https://github.com/joshdick/weeprowl>
#
# Requires WeeChat version 0.3.7 or greater
# Released under GNU GPL v2
#
# Based on the 'notify' plugin version 0.0.5 by lavaramano <lavaramano AT gmail DOT com>:
# <http://www.weechat.org/scripts/source/stable/notify.py.html/>
#
# 2021-03-03, Johannes Rabenschlag <j.rabenschlag@gmail.com>
#     Version 0.8: Fixed urllib error
# 2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#     Version 0.7: Change hook_print callback argument type of
#                  displayed/highlight (WeeChat >= 1.0)
# 2013-12-22, Josh Dick <josh@joshdick.net>
#     Version 0.6: Fixed bug that was preventing negative numbers from working with
#                  the prowl_priority setting
# 2013-12-20, Josh Dick <josh@joshdick.net>
#     Version 0.5: Now backgrounds Prowl API requests, added prowl_priority setting,
#                  now requires WeeChat version 0.3.7 or greater
# 2013-08-13, Josh Dick <josh@joshdick.net>
#     Version 0.4: No longer sending notifications for text you send in private messages
# 2012-09-16, Josh Dick <josh@joshdick.net>
#     Version 0.3: Removed 'smart_notification' and away_notification' settings
#                  in favor of more granular notification settings
# 2012-09-16, Josh Dick <josh@joshdick.net>
#     Version 0.2: Added 'away_notification' setting
# 2012-03-25, Josh Dick <josh@joshdick.net>
#     Version 0.1: Initial release

import urllib.parse, weechat

weechat.register('weeprowl', 'Josh Dick', '0.8', 'GPL', 'Prowl notifications for WeeChat', '', '')

# Plugin settings
settings = {
    'prowl_api_key': '',
    'prowl_priority': '0', # An integer value ranging [-2, 2] per http://www.prowlapp.com/api.php#add
    'show_hilights': 'on',
    'show_priv_msg': 'on',
    'nick_separator': ': ',
    'notify_focused_active': 'on', # If 'on', send Prowl notifications for the currently-focused buffer when not away
    'notify_focused_away': 'on', # If 'on', send Prowl notifications for the currently-focused buffer when away
    'notify_unfocused_active': 'on', # If 'on', send Prowl notifications for non-focused buffers when not away
    'notify_unfocused_away': 'on' # If 'on', send Prowl notifications for non-focused buffers when away
}

# Hook for private messages/hilights
weechat.hook_print('', 'irc_privmsg', '', 1, 'notification_callback', '')

# Shows an error/help message if prowl_api_key is not set
def show_config_help():
    weechat.prnt('', '%sweeprowl - Error: Your Prowl API key is not set!' % weechat.prefix('error'))
    weechat.prnt('', '%sweeprowl - To obtain a Prowl API key, visit <http://prowlapp.com>.' % weechat.prefix('error'))
    weechat.prnt('', '%sweeprowl - Once you have a Prowl API key, configure weeprowl to use it by running:' % weechat.prefix('error'))
    weechat.prnt('', '%sweeprowl - /set plugins.var.python.weeprowl.prowl_api_key "your_prowl_api_key_here"' % weechat.prefix('error'))

# Shows an error when there was a problem sending a Prowl notification.
def show_notification_error():
    weechat.prnt('', '%sweeprowl - Could not send Prowl notification.' % weechat.prefix('error'))

# Triggered by the weechat hook above
def notification_callback(data, bufferp, uber_empty, tagsn, isdisplayed, ishilight, prefix, message):

    is_away = weechat.buffer_get_string(bufferp, 'localvar_away')
    is_focused = bufferp == weechat.current_buffer()
    do_prowl = True # If set to False depending on state and settings, no Prowl notification will be sent

    if (is_away):
        if (is_focused and weechat.config_get_plugin('notify_focused_away') != 'on'):
            do_prowl = False
        elif (not is_focused and weechat.config_get_plugin('notify_unfocused_away') != 'on'):
            do_prowl = False
    else:
        if (is_focused and weechat.config_get_plugin('notify_focused_active') != 'on'):
            do_prowl = False
        elif (not is_focused and weechat.config_get_plugin('notify_unfocused_active') != 'on'):
            do_prowl = False

    if (do_prowl):
        if (weechat.buffer_get_string(bufferp, 'localvar_type') == 'private' and weechat.config_get_plugin('show_priv_msg') == 'on' and prefix != weechat.buffer_get_string(bufferp, 'localvar_nick')):
            send_prowl_notification(prefix, message, True)
        elif (int(ishilight) and weechat.config_get_plugin('show_hilights') == 'on'):
            buffer = (weechat.buffer_get_string(bufferp, 'short_name') or weechat.buffer_get_string(bufferp, 'name'))
            send_prowl_notification(buffer, prefix + weechat.config_get_plugin('nick_separator') + message, False)

    return weechat.WEECHAT_RC_OK

# Send a Prowl notification via the Prowl API (API documentation: <http://www.prowlapp.com/api.php>)
def send_prowl_notification(chan, message, isPrivate):

    # Make sure a Prowl API key has been configured
    prowl_api_key = weechat.config_get_plugin('prowl_api_key')
    if (prowl_api_key == ''):
        show_config_help()
        show_notification_error()
        return

    # Make sure a valid Prowl priority has been configured
    prowl_priority = weechat.config_get_plugin('prowl_priority')
    valid_prowl_priority = True
    try:
        if (int(prowl_priority) > 2 or int(prowl_priority) < -2):
            valid_prowl_priority = False
    except ValueError:
            valid_prowl_priority = False
    if (not valid_prowl_priority):
        weechat.prnt('', '%sweeprowl - Current prowl_priority setting "%s" is invalid.' % (weechat.prefix('error'), prowl_priority))
        weechat.prnt('', '%sweeprowl - Please set prowl_priority to an integer value ranging from [-2, 2].' % weechat.prefix('error'))
        show_notification_error()
        return

    # Build the Prowl API request parameters
    params = urllib.parse.urlencode({
        'apikey': prowl_api_key,
        'application': 'weechat',
        'event': 'IRC ' + 'Private Message' if isPrivate else 'Mention/Hilight',
        'description': 'Channel: ' + chan + '\n' + message,
        'priority': prowl_priority
    })

    # Build the complete Prowl API request URL
    prowl_api_url = 'https://api.prowlapp.com/publicapi/add?' + params

    # Make the Prowl API request
    weechat.hook_process_hashtable(
        'url:' + prowl_api_url,
        { 'post': '1' },
        30 * 1000,
        'send_prowl_notification_callback',
        ''
    )

# Callback that handles the result of the Prowl API request
def send_prowl_notification_callback(data, command, rc, stdout, stderr):

    # Show an error if the Prowl API request failed
    if (rc > 0):
        weechat.prnt('', '%sweeprowl - Error: There was a problem communicating with the Prowl API!' % weechat.prefix('error'))
        weechat.prnt('', '%sweeprowl - Prowl API response information:' % weechat.prefix('error'))
        weechat.prnt('', '%sweeprowl -     Response code = %s' % (weechat.prefix('error'), rc))
        weechat.prnt('', '%sweeprowl -     STDOUT = %s' % (weechat.prefix('error'), stdout))
        weechat.prnt('', '%sweeprowl -     STDERR = %s' % (weechat.prefix('error'), stderr))
        show_notification_error()

    return weechat.WEECHAT_RC_OK

# Initialization
for option, default_value in settings.items():
    if weechat.config_get_plugin(option) == '':
        weechat.config_set_plugin(option, default_value)

if (weechat.config_get_plugin('prowl_api_key') == ''):
    show_config_help()

# vim: autoindent expandtab smarttab shiftwidth=4




# Blackmore's Enhanced IRC-Notification Collection (BEINC)
# Copyright (C) 2013-2024 Simeon Simeonov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""BEINC client for Weechat"""

import datetime
import io
import json
import os
import ssl
import urllib.parse
import urllib.request

import weechat

__author__ = 'Simeon Simeonov'
__version__ = '4.4'
__license__ = 'GPL3'


enabled = True
global_values = {}

# few constants #
BEINC_POLICY_NONE = 0
BEINC_POLICY_ALL = 1
BEINC_POLICY_LIST_ONLY = 2
BEINC_CURRENT_CONFIG_VERSION = 2


class WeechatTarget:
    """
    The target (destination) class

    Each remote destination is represented as a WeechatTarget object
    """

    def __init__(self, target_dict):
        """
        :param target_dict: The config-dict node that represents this instance
        :type target_dict: dict
        """
        self._name = target_dict.get('name', '')
        if self._name == '':
            raise Exception('"name" not defined for target')
        self._url = target_dict.get('target_url', '')
        if self._url == '':
            raise Exception('"target_url" not defined for target')
        self._password = target_dict.get('target_password', '')
        self._pm_title_template = target_dict.get(
            'pm_title_template', '%s @ %S'
        )
        self._pm_message_template = target_dict.get(
            'pm_message_template', '%m'
        )
        self._cm_title_template = target_dict.get(
            'cm_title_template', '%c @ %S'
        )
        self._cm_message_template = target_dict.get(
            'cm_message_template', '%s -> %m'
        )
        self._nm_title_template = target_dict.get(
            'nm_title_template', '%c @ %S'
        )
        self._nm_message_template = target_dict.get(
            'nm_message_template', '%s -> %m'
        )
        self._chans = set(target_dict.get('channel_list', []))
        self._nicks = set(target_dict.get('nick_list', []))
        self._chan_messages_policy = int(
            target_dict.get('channel_messages_policy', BEINC_POLICY_LIST_ONLY)
        )
        self._priv_messages_policy = int(
            target_dict.get('private_messages_policy', BEINC_POLICY_ALL)
        )
        self._notifications_policy = int(
            target_dict.get('notifications_policy', BEINC_POLICY_ALL)
        )
        self._cert_file = target_dict.get('target_cert_file')
        self._timestamp_format = target_dict.get(
            'target_timestamp_format', '%H:%M:%S'
        )
        self._debug = bool(target_dict.get('debug', False))
        self._enabled = bool(target_dict.get('enabled', True))
        self._socket_timeout = int(target_dict.get('socket_timeout', 3))
        self._ssl_ciphers = target_dict.get('ssl_ciphers', '')
        self._disable_hostname_check = bool(
            target_dict.get('disable-hostname-check', False)
        )
        self._ssl_version = target_dict.get('ssl_version', 'auto')
        self._last_message = None  # datetime.datetime instance
        self._context = None
        self._context_setup()

    @property
    def name(self):
        """Target name (read-only property)"""
        return self._name

    @property
    def chans(self):
        """Target channel list (read-only property)"""
        return self._chans

    @property
    def nicks(self):
        """Target nick list (read-only property)"""
        return self._nicks

    @property
    def channel_messages_policy(self):
        """The target's channel messages policy (read-only property)"""
        return self._chan_messages_policy

    @property
    def private_messages_policy(self):
        """The target's private messages policy (read-only property)"""
        return self._priv_messages_policy

    @property
    def notifications_policy(self):
        """The target's notifications policy (read-only property)"""
        return self._notifications_policy

    @property
    def enabled(self):
        """The target's enabled status (bool property)"""
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        """The target's enabled status (bool property)"""
        self._enabled = value

    def __repr__(self):
        """repr() implementation"""
        last_message = 'never'
        if self._last_message is not None:
            last_message = self._last_message.strftime('%Y-%m-%d %H:%M:%S')
        return (
            f'name: {self._name}\nurl: {self._url}\n'
            f"enabled: {'yes' if self._enabled else 'no'}\n"
            f"channel_list: {', '.join(self._chans)}\n"
            f"nick_list: {', '.join(self._nicks)}\n"
            f'channel_messages_policy: {self._chan_messages_policy}\n'
            f'private_messages_policy: {self._priv_messages_policy}\n'
            f'notifications_policy: {self._notifications_policy}\n'
            f'last message: {last_message}\n'
            f'socket timeout: {self._socket_timeout}\n'
            f'ssl-version: {self._ssl_version}\n'
            f"ciphers: {self._ssl_ciphers or 'auto'}\n"
            "disable hostname check: "
            f"{'yes' if self._disable_hostname_check else 'no'}\n"
            f"debug: {'yes' if self._debug else 'no'}\n\n"
        )

    def send_private_message_notification(self, values):
        """
        Sends a private message notification to the represented target

        :param value: Dict pupulated by the irc msg-handler
        :type value: dict
        """
        try:
            title = self._fetch_formatted_str(self._pm_title_template, values)
            message = self._fetch_formatted_str(
                self._pm_message_template, values
            )
            if not self._send_beinc_message(title, message) and self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_private_message_notification-ERROR '
                    f'for "{self._name}": _send_beinc_message -> False'
                )
        except Exception as exp:
            if self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_private_message_notification-ERROR '
                    f'for "{self._name}": {exp}'
                )

    def send_channel_message_notification(self, values):
        """
        Sends a channel message notification to the represented target

        :param value: Dict pupulated by the irc msg-handler
        :type value: dict
        """
        try:
            title = self._fetch_formatted_str(self._cm_title_template, values)
            message = self._fetch_formatted_str(
                self._cm_message_template, values
            )
            if not self._send_beinc_message(title, message) and self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_channel_message_notification-ERROR '
                    f'for "{self._name}": _send_beinc_message -> False'
                )
        except Exception as exp:
            if self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_channel_message_notification-ERROR '
                    f'for "{self._name}": {exp}'
                )

    def send_notify_message_notification(self, values):
        """
        Sends a notify message notification to the represented target

        :param value: Dict pupulated by the irc msg-handler
        :type value: dict
        """
        try:
            title = self._fetch_formatted_str(self._nm_title_template, values)
            message = self._fetch_formatted_str(
                self._nm_message_template, values
            )
            if not self._send_beinc_message(title, message) and self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_notify_message_notification-ERROR '
                    f'for "{self._name}": _send_beinc_message -> False'
                )
        except Exception as exp:
            if self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_notify_message_notification-ERROR '
                    f'for "{self._name}": {exp}'
                )

    def send_broadcast_notification(self, message):
        """
        Sends a 'pure' broadcast / test message notification
        to the represented target

        :param message: A single message string
        :type message: str
        """
        try:
            title = 'BEINC broadcast'
            if not self._send_beinc_message(title, message) and self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_broadcast_notification-ERROR '
                    f'for "{self._name}": _send_beinc_message -> False'
                )
        except Exception as exp:
            if self._debug:
                beinc_prnt(
                    f'BEINC DEBUG: send_broadcast_notification-ERROR '
                    f'for "{self._name}": {exp}'
                )

    def _context_setup(self):
        """Sets up the SSL context"""
        if self._context is not None:
            return True
        try:
            context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
            if self._cert_file:
                context.verify_mode = ssl.CERT_REQUIRED
                context.load_verify_locations(
                    cafile=os.path.expanduser(self._cert_file)
                )
                context.check_hostname = bool(not self._disable_hostname_check)
            else:
                context.check_hostname = False
                context.verify_mode = ssl.CERT_NONE
            if self._ssl_ciphers and self._ssl_ciphers != 'auto':
                context.set_ciphers(self._ssl_ciphers)
            self._context = context
            return True
        except ssl.SSLError as err:
            if self._debug:
                beinc_prnt(f'BEINC DEBUG: SSL/TLS error: {err}\n')
        except Exception as exp:
            if self._debug:
                beinc_prnt(f'BEINC DEBUG: Generic context error: {exp}\n')
        self._context = None
        return False

    def _fetch_formatted_str(self, template, values):
        """
        Returns a formatted string by replacing the defined
        macros in 'template' the the corresponding values from 'values'

        :param template: The template to use
        :type template: str

        :param values: The values dict
        :type values: dict

        :return: The formatted string
        :rtype: str
        """
        timestamp = datetime.datetime.now().strftime(self._timestamp_format)
        replacements = {
            '%S': values['server'],
            '%s': values['source_nick'],
            '%c': values['channel'],
            '%m': values['message'],
            '%t': timestamp,
            '%p': 'BEINC',
            '%n': values['own_nick'],
        }
        for key, value in replacements.items():
            template = template.replace(key, value)
        return template

    def _send_beinc_message(self, title, message):
        """
        The method implements the BEINC "protocol" by generating a simple
        POST request

        :param title: The title
        :type title: str

        :param message: The message
        :type message: str

        :return: The status
        :rtype: bool
        """
        try:
            if self._context is None and not self._context_setup():
                return False
            response = urllib.request.urlopen(
                self._url,
                data=urllib.parse.urlencode(
                    (
                        ('resource_name', self._name),
                        ('password', self._password),
                        ('title', title),
                        ('message', message),
                    )
                ).encode('utf-8'),
                timeout=self._socket_timeout,
                context=self._context,
            )
            response_dict = json.loads(response.read().decode('utf-8'))
            if response.code != 200:
                raise OSError(response_dict.get('message', ''))
            if self._debug:
                beinc_prnt(
                    "BEINC DEBUG: Server responded: "
                    f"{response_dict.get('message')}"
                )
            self._last_message = datetime.datetime.now()
            return True
        except ssl.SSLError as err:
            if self._debug:
                beinc_prnt(f'BEINC DEBUG: SSL/TLS error: {err}\n')
        except OSError as err:
            if self._debug:
                beinc_prnt(f'BEINC DEBUG: Connection error: {err}\n')
        except Exception as exp:
            if self._debug:
                beinc_prnt(f'BEINC DEBUG: Unable to send message: {exp}\n')
        return False


def beinc_prnt(message_str):
    """wrapper around weechat.prnt"""
    if global_values['use_current_buffer']:
        weechat.prnt(weechat.current_buffer(), message_str)
    else:
        weechat.prnt('', message_str)


def beinc_cmd_broadcast_handler(cmd_tokens):
    """handles: '/beinc broadcast' command actions"""
    if not cmd_tokens:
        beinc_prnt('beinc broadcast <message>')
        return weechat.WEECHAT_RC_OK
    for target in target_list:
        if target.enabled:
            target.send_broadcast_notification(' '.join(cmd_tokens))
    return weechat.WEECHAT_RC_OK


def beinc_cmd_target_handler(cmd_tokens):
    """handles: '/beinc target' command actions"""
    if not cmd_tokens or cmd_tokens[0] not in ['list', 'enable', 'disable']:
        beinc_prnt('beinc target [ list | enable <name> | disable <name> ]')
        return weechat.WEECHAT_RC_OK
    if cmd_tokens[0] == 'list':
        beinc_prnt('--- Globals ---')
        for key, value in global_values.items():
            beinc_prnt(f'{key} -> {value}')
        beinc_prnt('--- Targets ---')
        for target in target_list:
            beinc_prnt(str(target))
        beinc_prnt('---------------')
    elif cmd_tokens[0] == 'enable':
        if not cmd_tokens[1:]:
            beinc_prnt('missing a name-argument')
            return weechat.WEECHAT_RC_OK
        name = ' '.join(cmd_tokens[1:])
        for target in target_list:
            if target.name == name:
                target.enabled = True
                beinc_prnt(f'target "{name}" enabled')
                break
        else:
            beinc_prnt(f'no matching target for "{name}"')
    elif cmd_tokens[0] == 'disable':
        if not cmd_tokens[1:]:
            beinc_prnt('missing a name-argument')
            return weechat.WEECHAT_RC_OK
        name = ' '.join(cmd_tokens[1:])
        for target in target_list:
            if target.name == name:
                target.enabled = False
                beinc_prnt(f'target "{name}" disabled')
                break
        else:
            beinc_prnt(f'no matching target for "{name}"')
    return weechat.WEECHAT_RC_OK


def beinc_command(data, buffer_obj, args):
    """Callback function handling the Weechat's /beinc command"""
    global enabled
    cmd_tokens = args.split()
    if not cmd_tokens:
        return weechat.WEECHAT_RC_OK
    if args == 'on':
        enabled = True
        beinc_prnt('BEINC on')
    elif args == 'off':
        enabled = False
        beinc_prnt('BEINC off')
    elif args == 'reload':
        beinc_prnt('Reloading BEINC...')
        beinc_init()
    elif cmd_tokens[0] in ('broadcast', 'test'):
        return beinc_cmd_broadcast_handler(cmd_tokens[1:])
    elif cmd_tokens[0] == 'target':
        return beinc_cmd_target_handler(cmd_tokens[1:])
    else:
        beinc_prnt(
            'syntax: /beinc < on | off | reload |'
            ' broadcast <text> | target <action> >'
        )
    return weechat.WEECHAT_RC_OK


def beinc_privmsg_handler(data, signal, signal_data):
    """Callback function the *PRIVMSG* IRC messages hooked by Weechat"""
    if not enabled:
        return weechat.WEECHAT_RC_OK
    prvmsg_dict = weechat.info_get_hashtable(
        'irc_message_parse', {'message': signal_data}
    )
    # packing the privmsg handler values
    ph_values = {}
    ph_values['server'] = signal.split(',')[0]
    ph_values['own_nick'] = weechat.info_get('irc_nick', ph_values['server'])
    ph_values['channel'] = prvmsg_dict['arguments'].split(':')[0].strip()
    ph_values['source_nick'] = prvmsg_dict['nick']
    ph_values['message'] = ':'.join(
        prvmsg_dict['arguments'].split(':')[1:]
    ).strip()
    if ph_values['channel'] == ph_values['own_nick']:
        # priv messages are handled here
        if not global_values['global_private_messages_policy']:
            return weechat.WEECHAT_RC_OK
        for target in target_list:
            if not target.enabled:
                continue
            p_messages_policy = target.private_messages_policy
            if p_messages_policy == BEINC_POLICY_ALL or (
                p_messages_policy == BEINC_POLICY_LIST_ONLY
                and f"{ph_values['server']}.{ph_values['source_nick'].lower()}"
                in target.nicks
            ):
                target.send_private_message_notification(ph_values)
    elif ph_values['own_nick'].lower() in ph_values['message'].lower():
        # notify messages are handled here
        if not global_values['global_notifications_policy']:
            return weechat.WEECHAT_RC_OK
        for target in target_list:
            if not target.enabled:
                continue
            if target.notifications_policy == BEINC_POLICY_ALL or (
                target.notifications_policy == BEINC_POLICY_LIST_ONLY
                and f"{ph_values['server']}.{ph_values['channel'].lower()}"
                in target.chans
            ):
                target.send_notify_message_notification(ph_values)
    elif global_values['global_channel_messages_policy']:
        # chan messages are handled here
        if not global_values['global_notifications_policy']:
            return weechat.WEECHAT_RC_OK
        for target in target_list:
            if not target.enabled:
                continue
            c_messages_policy = target.channel_messages_policy
            if c_messages_policy == BEINC_POLICY_ALL or (
                c_messages_policy == BEINC_POLICY_LIST_ONLY
                and f"{ph_values['server']}.{ph_values['channel'].lower()}"
                in target.chans
            ):
                target.send_channel_message_notification(ph_values)
    return weechat.WEECHAT_RC_OK


def beinc_init():
    """
    Ran every time the script is (re)loaded
    It loads the config (.json) file and (re)loads its contents into memory
    beinc_init() will disable all notifications on failure
    """

    global enabled
    global target_list
    global global_values

    # global chans/nicks sets are used to speed up the filtering
    global_values = {}
    target_list = []
    custom_error = ''
    global_values['global_channel_messages_policy'] = False
    global_values['global_private_messages_policy'] = False
    global_values['global_notifications_policy'] = False
    global_values['use_current_buffer'] = False

    try:
        beinc_config_file_str = os.path.join(
            weechat.info_get('weechat_dir', ''), 'beinc_weechat.json'
        )
        beinc_prnt(f'Parsing {beinc_config_file_str}...')
        custom_error = 'load error'
        with io.open(beinc_config_file_str, 'r', encoding='utf-8') as fp:
            config_dict = json.load(fp)
        custom_error = 'target parse error'
        global_values['use_current_buffer'] = bool(
            config_dict['irc_client'].get('use_current_buffer', False)
        )
        if (
            config_dict.get('config_version', 0)
            != BEINC_CURRENT_CONFIG_VERSION
        ):
            beinc_prnt(
                "WARNING: The version of the config-file: "
                f"{beinc_config_file_str} "
                f"({config_dict.get('config_version', 0)}) "
                "does not correspond to the latest version supported "
                f"by this program ({BEINC_CURRENT_CONFIG_VERSION})\n"
                "Check beinc_config_sample.json for the newest features!"
            )
        for target in config_dict['irc_client']['targets']:
            try:
                new_target = WeechatTarget(target)
            except Exception as exp:
                beinc_prnt(f'Unable to add target: {exp}')
                continue
            if new_target.channel_messages_policy:
                global_values['global_channel_messages_policy'] = True
            if new_target.private_messages_policy:
                global_values['global_private_messages_policy'] = True
            if new_target.notifications_policy:
                global_values['global_notifications_policy'] = True
            target_list.append(new_target)
            beinc_prnt(f'BEINC target "{new_target.name}" added')
        beinc_prnt('Done!')
    except Exception as exp:
        beinc_prnt(
            f'ERROR: unable to parse {beinc_config_file_str}: '
            f'{custom_error} - {exp}\nBEINC is now disabled'
        )
        enabled = False
        # do not return error / exit the script
        # in order to give a smoother opportunity to fix a 'broken' config
        return weechat.WEECHAT_RC_OK
    return weechat.WEECHAT_RC_OK


weechat.register(
    'beinc_weechat',
    __author__,
    __version__,
    __license__,
    "Blackmore's Extended IRC Notification Collection (Weechat Client)",
    '',
    '',
)
version = weechat.info_get('version_number', '') or 0
if int(version) < 0x00040000:
    weechat.prnt('', 'WeeChat version >= 0.4.0 is required to run beinc')
else:
    weechat.hook_command(
        'beinc',
        'BEINC command',
        '< broadcast <message> | on | off | reload | target <action> >',
        (
            'Available target actions:\n'
            'disable <target name>\nenable <target name>\nlist'
        ),
        'None',
        'beinc_command',
        '',
    )
    weechat.hook_signal('*,irc_in2_privmsg', 'beinc_privmsg_handler', '')
    beinc_init()
    weechat.prnt('', 'beinc initiated!')



#
# SPDX-FileCopyrightText: 2021-2026 Sébastien Helleu <flashcode@flashtux.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# Prevent a password from being accidentally sent to a buffer
# (requires WeeChat ≥ 0.4.0 and WeeChat ≥ 3.1 to check secured data).
#

"""Anti password script."""

import re

IMPORT_OK = True

try:
    import weechat  # type: ignore[import]
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org/")
    IMPORT_OK = False

SCRIPT_NAME = "anti_password"
SCRIPT_AUTHOR = "Sébastien Helleu <flashcode@flashtux.org>"
SCRIPT_VERSION = "1.2.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Prevent a password from being accidentally sent to a buffer"

# script options
AP_SETTINGS_DEFAULT = {
    "allowed_regex": {
        "default": "^(http|https|ftp|file|irc)://",
        "help": (
            "allowed regular expression (case is ignored); this is checked "
            "first: if the input matched this regular expression, it is "
            "considered safe and sent immediately to the buffer; "
            "if empty, nothing specific is allowed"
        ),
    },
    "password_condition": {
        "default": (
            "${words} == 1 && ${lower} >= 1 && ${upper} >= 1 "
            "&& ${digits} >= 1 && ${special} >= 1"
        ),
        "help": (
            "condition evaluated to check if the input string is a password; "
            "allowed variables: "
            "${words} = number of words, "
            "${lower} = number of lower case letters, "
            "${upper} = number of upper case letters, "
            "${digits} = number of digits, "
            "${special} = number of other chars (not letter/digits/spaces), "
        ),
    },
    "check_secured_data": {
        "default": "equal",
        "help": (
            "consider that all secured data values are passwords and can not "
            "be sent to buffers, possible values: "
            "off = do not check secured data at all, "
            "equal = reject input if stripped input is equal to a secured "
            "data value, "
            "include = reject input if a secured data value is part of input"
        ),
    },
    "max_rejects": {
        "default": "3",
        "help": (
            "max number of rejects for a given input text; if you press Enter "
            "more than N times with exactly the same input, the text is "
            "finally sent to the buffer; "
            "if set to 0, the input is never sent to the buffer when it is "
            "considered harmful (be careful, according to the other settings, "
            "this can completely block the input)"
        ),
    },
}
ap_settings = {}
ap_reject = {
    "input": "",
    "count": 0,
}


def ap_config_cb(_data: str, option: str, value: str) -> int:
    """Called when a script option is changed."""
    pos = option.rfind(".")
    if pos > 0:
        name = option[pos + 1 :]
        if name in ap_settings:
            ap_settings[name] = value
    return weechat.WEECHAT_RC_OK


def ap_input_is_secured_data(input_text: str) -> bool:
    """Check if input_text is any value of a secured data."""
    check = ap_settings["check_secured_data"]
    if check == "off":
        return False
    sec_data = weechat.info_get_hashtable("secured_data", {}) or {}
    if check == "include":
        for value in sec_data.values():
            if value and value in input_text:
                return True
    if check == "equal":
        return input_text.strip() in sec_data.values()
    return False


def ap_input_matches_condition(input_text: str) -> bool:
    """Check if input_text matches the password condition."""
    # count chars in the input text
    words = len(list(filter(None, re.split(r"\s+", input_text))))
    lower = sum(1 for c in input_text if c.islower())
    upper = sum(1 for c in input_text if c.isupper())
    digits = sum(1 for c in input_text if c.isdigit())
    special = sum(1 for c in input_text if not (c.isalnum() or c.isspace()))

    # evaluate password condition
    extra_vars = {
        "words": str(words),
        "lower": str(lower),
        "upper": str(upper),
        "digits": str(digits),
        "special": str(special),
    }
    ret = weechat.string_eval_expression(
        ap_settings["password_condition"],
        {},
        extra_vars,
        {"type": "condition"},
    )
    return ret == "1"


def ap_input_is_password(input_text: str) -> bool:
    """Check if input_text looks like a password."""
    return ap_input_is_secured_data(input_text) or ap_input_matches_condition(
        input_text
    )


def ap_input_return_cb(_data: str, buf: str, _command: str) -> int:
    """Callback called when Return key is pressed in a buffer."""
    try:
        max_rejects = int(ap_settings["max_rejects"])
    except ValueError:
        max_rejects = 3

    input_text = weechat.buffer_get_string(buf, "input")

    # check if input is a command
    if not weechat.string_input_for_buffer(input_text):
        # commands are ignored
        ap_reject["input"] = ""
        ap_reject["count"] = 0
        return weechat.WEECHAT_RC_OK

    # check if input matches the allowed regex
    regex = ap_settings["allowed_regex"]
    if regex and re.search(regex, input_text, re.IGNORECASE):
        # allowed regex
        ap_reject["input"] = ""
        ap_reject["count"] = 0
        return weechat.WEECHAT_RC_OK

    if ap_input_is_password(input_text):
        if ap_reject["input"] == input_text:
            if ap_reject["count"] >= max_rejects > 0:
                # it looks like a password but send anyway after N rejects
                ap_reject["input"] = ""
                ap_reject["count"] = 0
                return weechat.WEECHAT_RC_OK
            ap_reject["count"] += 1
        else:
            ap_reject["input"] = input_text
            ap_reject["count"] = 1
        # password detected, do NOT send it to the buffer!
        return weechat.WEECHAT_RC_OK_EAT

    # not a password
    ap_reject["input"] = ""
    ap_reject["count"] = 0
    return weechat.WEECHAT_RC_OK


def main() -> None:
    """Main function."""
    if weechat.register(
        SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""
    ):
        # set default settings
        for name, option in AP_SETTINGS_DEFAULT.items():
            if weechat.config_is_set_plugin(name):
                ap_settings[name] = weechat.config_get_plugin(name)
            else:
                weechat.config_set_plugin(name, option["default"])
                ap_settings[name] = option["default"]
            weechat.config_set_desc_plugin(
                name, f'{option["help"]} (default: "{option["default"]}")'
            )

        # detect config changes
        weechat.hook_config(f"plugins.var.python.{SCRIPT_NAME}.*", "ap_config_cb", "")

        # hook Return key
        weechat.hook_command_run("/input return", "ap_input_return_cb", "")


if __name__ == "__main__" and IMPORT_OK:
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2011 by agreeabledragon <recognize@me.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# (this script requires Spotify for Mac v0.5.1.98 or newer)
#
# History:
#
# 2022-01-25, Sébastien Helleu <flashcode@flashtux.org>
#     version 0.1.2: fix mixed spaces and tabs for indentation
#
# 2011-06-12, agreeabledragon <recognize@me.com>
#     version 0.1.1: rewrote it to use weechat.hook_process() to prevent it from blocking weechat as requested by Sébastien
#
# 2011-06-12, agreeabledragon <recognize@me.com>
#     version 0.1: initial release
#
# @TODO: add options for customizing the output
import weechat as w, re, subprocess, sys

SCRIPT_NAME    = "spotify_nowplaying"
SCRIPT_AUTHOR  = "agreeabledragon <recognize@me.com>"
SCRIPT_VERSION = "0.1.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Current song script for Spotify (v0.5.1.98 or newer) on OS X"
SCRIPT_COMMAND = "spotify"
# For executing the script
SCRIPT_TIMEOUT = 1500
SCRIPT_PROCESS = False
SCRIPT_BUFFER  = False

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", "") and sys.platform == "darwin":
    w.hook_command(SCRIPT_COMMAND,
                   SCRIPT_DESC,
                   "",
                   "",
                   "",
                   "spotify_exec",
                   "")
else:
    w.prnt("", "WARNING: This now playing script for Spotify only works on OS X with Spotify version 0.5.1.98 (or newer)")

def spotify_process(data, command, rc, stdout, stderr):
    global SCRIPT_BUFFER, SCRIPT_PROCESS
    if stderr:
        w.prnt("", "There was an error executing the script - make sure you meet the requirements (OS X with Spotify v0.5.1.98 or newer)")
        SCRIPT_BUFFER  = False
        SCRIPT_PROCESS = False
        return w.WEECHAT_RC_ERROR
    else:
        w.command(SCRIPT_BUFFER, stdout)
        SCRIPT_BUFFER  = False
        SCRIPT_PROCESS = False
        return w.WEECHAT_RC_OK

def spotify_exec(data, buffer, args):
    global SCRIPT_TIMEOUT, SCRIPT_BUFFER, SCRIPT_PROCESS
    if SCRIPT_PROCESS:
        w.prnt("", "Please wait for the other command to finish")
        return w.WEECHAT_RC_ERROR
    else:
        script = """set AppleScript's text item delimiters to ASCII character 10
                    set spotify_active to false
                    set theString to \\"/me is not currently running Spotify.\\"

                    tell application \\"Finder\\"
                        if (get name of every process) contains \\"Spotify\\" then set spotify_active to true
                    end tell

                    if spotify_active then
                        set got_track to false

                        tell application \\"Spotify\\"
                            if player state is playing then
                                set theTrack to name of the current track
                                set theArtist to artist of the current track
                                set theAlbum to album of the current track
                                set isStarred to starred of the current track
                                set got_track to true
                            end if
                        end tell

                        set theString to \\"/me is not playing anything in Spotify.\\"

                        if got_track then
                            if isStarred then
                                set theString to \\"/me is listening to one of my favorite tracks \\\\\\"\\" & theTrack & \\"\\\\\\" by \\" & theArtist & \\" (Album: \\" & theAlbum & \\")\\"
                            else
                                set theString to \\"/me is listening to \\\\\\"\\" & theTrack & \\"\\\\\\" by \\" & theArtist & \\" (Album: \\" & theAlbum & \\")\\"
                            end if
                        end if
                    end if

                    return theString"""
        SCRIPT_BUFFER = buffer
        SCRIPT_PROCESS = w.hook_process('arch -i386 osascript -e "' + script + '"', SCRIPT_TIMEOUT, "spotify_process", "")
        return w.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (c) 2011 by Richard A Hofer (javagamer) <atomic.quark@gmail.com>
#
# fileaway.py - A simple autoaway script for Weechat which monitors a file,
# allowing it to easily connect to external things (such as xscreensaver)
#
# The code from screen_away.py and auto_away.py were heavily consulted in the
# writing of this script
# ---------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -------------------------------------------------------------------------
# Purpose -
# I wrote this auto-away script so that it could easily be hooked into other
# things to deterine whether or not I was there rather than just watching for
# keyboard input in Weechat.  In my case I wanted to be able to have weechat in
# a tmux session on a server, yet still go away when I lock the screen on my
# desktop.
#
# Below is a sample shellscript which watches xscreensaver and sets away when
# it is locked, and available when unlocked
# While this only one way this script can be used, this is why I wrote it
#
# #!/bin/sh
#
# # Read xscreensaver's state
# xscreensaver-command -watch|
# while read STATUS; do
#   case "$STATUS" in
#     LOCK*)
#       rm ~/.available
#       ;;
#     UNBLANK*)
#       touch ~/.available
#       ;;
#   esac
# done
#
# Alternative for xset:
#
# #!/bin/bash
#
# xset -display ":0" q|grep "Monitor"|awk ' { print $3 $4 } '|
# while read MONITOR; do
#    case "$MONITOR" in
#      Off)
#        rm ~/.available
#        ;;
#      On)
#        touch ~/.available
#        ;;
#    esac
# done
# ------------------------------------------------------------------------
# Changelog:
# Version 1.0 released - March 27, 2011
#  -Initial release
# Version 1.0.1 released - March 31, 2011
#  -Handles improper commands
# Version 1.0.2 release - Jun 15, 2011
#  -Added alternative for xset users (credit: sherpa9 at irc.freenode.net)
# Version 1.0.3 release - Dec 19, 2019
#  -Adapt for python3

from __future__ import print_function

try:
  import weechat as w
  import os.path

except Exception:
  print("This script must be run under WeeChat.")
  print("Get WeeChat now at: http://www.wwchat.org/")
  quit()

SCRIPT_NAME     = "fileaway"
SCRIPT_AUTHOR   = "javagamer"
SCRIPT_VERSION  = "1.0.3"
SCRIPT_LICENSE  = "GPL3"
SCRIPT_DESC     = "Set away status based on presence of a file"
debug           = 0
TIMER           = None

settings = {
  'filepath':   './.available',
  'awaymessage':'Away',
  'interval':   '20', # How often to check for inactivity (in seconds)
  'status':     '0',
  'away':       '0',
}

def set_back(overridable_messages):
  '''Removes away status for servers where one of the overridable_messages is set'''
  if(w.config_get_plugin('away') == '0'): return # No need to come back again
  serverlist = w.infolist_get('irc_server','','')
  if serverlist:
    buffers = []
    while w.infolist_next(serverlist):
      if w.infolist_string(serverlist, 'away_message') in overridable_messages:
        ptr = w.infolist_pointer(serverlist, 'buffer')
        if ptr:
          buffers.append(ptr)
    w.infolist_free(serverlist)
    for buffer in buffers:
      w.command(buffer, "/away")
  w.config_set_plugin('away', '0')

def set_away(message, overridable_messages=[]):
  '''Sets away status, but respectfully (so it doesn't change already set statuses'''
  if(w.config_get_plugin('away') == '1'): return # No need to go away again (this prevents some repeated messages)
  if(debug): w.prnt('', "Setting away to %s" % message)
  serverlist = w.infolist_get('irc_server','','')
  if serverlist:
    buffers = []
    while w.infolist_next(serverlist):
      if w.infolist_integer(serverlist, 'is_away') == 0:
        if(debug): w.prnt('', "Not away on %s" % w.infolist_string(serverlist, 'name'))
        ptr = w.infolist_pointer(serverlist, 'buffer')
        if ptr:
          buffers.append(ptr)
      elif w.infolist_string(serverlist, 'away_message') in overridable_messages:
        if(debug): w.prnt('', "%s is in %s" % (w.infolist_string(serverlist, 'away_message'), repr(overridable_messages)))
        buffers.append(w.infolist_pointer(serverlist, 'buffer'))
    w.infolist_free(serverlist)
    if(debug): w.prnt('', repr(buffers))
    for buffer in buffers:
      w.command(buffer, "/away %s" % message)
  w.config_set_plugin('away', '1')

def fileaway_cb(data, buffer, args):
  response = {  'enable'  : lambda args: w.config_set_plugin('status', '1') and check_timer(),
                'disable' : lambda args: w.config_set_plugin('status', '0') and check_timer(),
                'check'   : check,
                'file'    : lambda filepath: w.config_set_plugin('filepath', filepath),
                'interval': lambda interval: w.config_set_plugin('interval', interval),
                'msg'     : lambda status: w.config_set_plugin('awaymessage', status),
            }
  if args:
    words = args.strip().partition(' ')
    if words[0] in response:
      response[words[0]](words[2])
    else:
      w.prnt('', "Fileaway error: %s not a recognized command.  Try /help fileaway" % words[0])
  w.prnt('', "fileaway: enabled: %s interval: %s away message: \"%s\" filepath: %s" %
    (w.config_get_plugin('status'), w.config_get_plugin('interval'),
    w.config_get_plugin('awaymessage'), w.config_get_plugin('filepath')))
  return w.WEECHAT_RC_OK

def auto_check(data, remaining_calls):
  '''Callback from timer'''
  check(0)
  return w.WEECHAT_RC_OK

def check(args):
  '''Check for existance of file and set away if it isn't there'''
  if os.path.isfile(w.config_get_plugin('filepath')):
    set_back([w.config_get_plugin('awaymessage')])
  else:
    set_away(w.config_get_plugin('awaymessage'), [])

def check_timer():
  '''Sets or unsets the timer based on whether or not the plugin is enabled'''
  global TIMER
  if TIMER:
      w.unhook(TIMER)
  if w.config_get_plugin('status') == '1':
    TIMER = w.hook_timer(int(w.config_get_plugin('interval')) * 1000, 0, 0, "auto_check", "")
    w.prnt('', "fileaway timer is running.")

if __name__ == "__main__":
  if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
      if not w.config_is_set_plugin(option):
        w.config_set_plugin(option, default_value)

    w.hook_command("fileaway",
    "Set away status based on presense or absense of a file.",
    "check, msg [status], interval [time], file [filepath], or enable|disable",
    "check - manually checks for file rather than waiting for interval.\n"
    "msg [status] - sets the away message"
    "interval [time] - sets the interval to check for the file.\n"
    "file [filepath] - sets the file to be watched.\n"
    "enable|disable - enables or disables plugin.\n",
    "check"
    " || msg"
    " || interval"
    " || file %(filename)"
    " || enable"
    " || disable",
    "fileaway_cb", "")
    check_timer()
    if(w.config_get_plugin('status') == '0'):
      w.prnt('', "fileaway is currently disabled.  Type /fileaway enable to enable it.")
    else:
      w.prnt('', "fileaway is currently enabled.  Type /fileaway disable to disable it.")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2011-2012 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Flood'it game for WeeChat.
# (mouse supported with WeeChat >= 0.3.6)
#
# History:
#
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.4: make script compatible with Python 3.x
# 2011-09-29, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.3: fix error on floodit buffer after /upgrade
# 2011-08-20, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.2: add "q" (or "quit") to close floodit buffer
# 2011-08-20, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
#

SCRIPT_NAME    = 'floodit'
SCRIPT_AUTHOR  = 'Sebastien Helleu <flashcode@flashtux.org>'
SCRIPT_VERSION = '0.4'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC    = 'Flood\'it game'

SCRIPT_COMMAND = 'floodit'

import_ok = True

try:
    import weechat
except ImportError:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: http://www.weechat.org/')
    import_ok = False

try:
    import random, copy
except ImportError as message:
    print('Missing package(s) for %s: %s' % (SCRIPT_NAME, message))
    import_ok = False

floodit = {
    'buffer'   : '',
    'mode'     : 'single',
    'board'    : [],
    'sizes'    : { 14: 25, 21: 35, 28: 50 },
    'size'     : 14,
    'zoom'     : 1,
    'colors'   : [],
    'color'    : 0,
    'count'    : 0,
    'count_max': 25,
    'end'      : '',
    'timer'    : '',
}

# script options
floodit_settings_default = {
    'colors'           : ['blue,red,green,yellow,magenta,cyan', 'comma-separated list of 6 colors for squares'],
    'zoom'             : ['',                                   'zoom for board (0-N, empty means automatic zoom according to size of window)'],
}
floodit_settings = {}

# mouse keys
floodit_mouse_keys = { '@chat(python.floodit):button1': '/window ${_window_number};hsignal:floodit_mouse' }

def floodit_display(clear=False):
    """Display status and board."""
    global floodit
    if not floodit['buffer']:
        return
    if clear:
        weechat.buffer_clear(floodit['buffer'])
    spaces = ' ' * ((floodit['zoom'] + 1) * 2)
    str_line = ''
    for index, color in enumerate(floodit['colors']):
        str_select = [' ', ' ']
        if floodit['color'] == index:
            str_select = ['»', '«']
        str_line += '%s%s%s%s%s%s%s' % (weechat.color('white,default'),
                                        str_select[0],
                                        weechat.color(',%s' % color),
                                        spaces,
                                        weechat.color('white,default'),
                                        str_select[1],
                                        spaces[0:-2])
    str_status = ''
    str_end = ''
    if floodit['mode'] == 'single':
        board = copy.deepcopy(floodit['board'])
        floodit_flood_xy(board, 0, 0, board[0][0])
        percent = (floodit_count_color(board, -1) * 100) // (floodit['size'] * floodit['size'])
        str_status = '%2d/%d%s (%d%%)' % (floodit['count'], floodit['count_max'],
                                          weechat.color('chat'), percent)
        message_end = { 'win': '** CONGRATS! **', 'lose': '...GAME OVER!...' }
    elif floodit['mode'] == 'versus':
        colors = ['yellow', 'lightred']
        board = copy.deepcopy(floodit['board'])
        floodit_flood_xy(board, 0, 0, board[0][0])
        count_player = floodit_count_color(board, -1)
        board = copy.deepcopy(floodit['board'])
        floodit_flood_xy(board, floodit['size'] - 1, floodit['size'] - 1, board[floodit['size'] - 1][floodit['size'] - 1])
        count_computer = floodit_count_color(board, -1)
        if count_player == count_computer:
            colors[1] = 'yellow'
        elif count_computer > count_player:
            colors.reverse()
        str_status = '%sYou: %d%s / %sWee: %d' % (weechat.color(colors[0]), count_player,
                                                  weechat.color('default'),
                                                  weechat.color(colors[1]), count_computer)
        message_end = { 'win': '** YOU WIN! **', 'lose': '...You lose...', 'equality': 'Equality!' }
    str_end = '%s%s' % (weechat.color('white'), message_end.get(floodit['end'], ''))
    weechat.prnt_y(floodit['buffer'], 0, '%s %s %s' % (str_line, str_status, str_end))
    for i in range (0, floodit['zoom']):
        weechat.prnt_y(floodit['buffer'], 1 + i, str_line)
    weechat.prnt_y(floodit['buffer'], floodit['zoom'] + 1, '%s%s' % (weechat.color('blue'), '─' * (floodit['size'] * ((floodit['zoom'] + 1) * 2))))
    for y, line in enumerate(floodit['board']):
        str_line = ''
        for color in line:
            str_line += '%s%s' % (weechat.color(',%s' % floodit['colors'][color]), spaces)
        str_line += '%s' % weechat.color('chat')
        for i in range (0, floodit['zoom'] + 1):
            weechat.prnt_y(floodit['buffer'], floodit['zoom'] + 2 + (y * (floodit['zoom'] + 1)) + i, str_line)

def floodit_adjust_zoom():
    """Choose zoom according to size of window."""
    global floodit, floodit_settings
    floodit['zoom'] = -1
    if floodit_settings['zoom']:
        try:
            floodit['zoom'] = int(floodit_settings['zoom'])
        except:
            floodit['zoom'] = -1
    if floodit['zoom'] < 0:
        width = weechat.window_get_integer(weechat.current_window(), 'win_chat_width')
        height = weechat.window_get_integer(weechat.current_window(), 'win_chat_height')
        for i in range(10, -1, -1):
            if width >= floodit['size'] * ((i + 1) * 2) and height >= (floodit['size'] * (i + 1)) + i + 2:
                floodit['zoom'] = i
                break
    if floodit['zoom'] < 0:
        floodit['zoom'] = 0

def floodit_set_colors():
    """Set list of colors using settings."""
    global floodit, floodit_settings, floodit_settings_default
    floodit['colors'] = floodit_settings['colors'].split(',')
    if len(floodit['colors']) != 6:
        weechat.prnt('', '%sfloodit: invalid colors (list must have 6 colors)' % weechat.prefix('error'))
        floodit['colors'] = floodit_settings_default['colors'][0].split(',')

def floodit_config_cb(data, option, value):
    """Called when a script option is changed."""
    global floodit_settings
    pos = option.rfind('.')
    if pos > 0:
        name = option[pos+1:]
        if name in floodit_settings:
            floodit_settings[name] = value
            if name == 'colors':
                floodit_set_colors()
            elif name == 'zoom':
                floodit_adjust_zoom()
    floodit_display()
    return weechat.WEECHAT_RC_OK

def floodit_new_game():
    """Create a new game: initialize board and some variables."""
    global floodit
    floodit['board'] = []
    for y in range(0, floodit['size']):
        line = []
        for x in range(0, floodit['size']):
            line.append(random.randint(0, 5))
        floodit['board'].append(line)
    if floodit['mode'] == 'versus':
        floodit['board'][floodit['size'] - 1][floodit['size'] - 1] = floodit['board'][0][0]
    floodit['color'] = 0
    floodit['count'] = 0
    floodit['end'] = ''
    floodit_display()

def floodit_change_size(add):
    """Change size of board."""
    global floodit
    keys = sorted(floodit['sizes'])
    index = keys.index(floodit['size']) + add
    if index >= 0 and index < len(keys):
        floodit['size'] = keys[index]
        floodit['count_max'] = floodit['sizes'][floodit['size']]
        weechat.buffer_clear(floodit['buffer'])
        floodit_adjust_zoom()
        floodit_new_game()

def floodit_timer_cb(data, remaining_calls):
    """Timer for demo mode."""
    global floodit
    floodit['color'] = floodit_find_best(0, 0)
    floodit_user_flood()
    if floodit['end']:
        weechat.unhook(floodit['timer'])
        floodit['timer'] = ''
    return weechat.WEECHAT_RC_OK

def floodit_input_buffer(data, buffer, input):
    """Input data in floodit buffer."""
    global floodit
    if input:
        args = input.split(' ')
        if args[0] in ('d', 'demo'):
            if not floodit['timer']:
                delay = 500
                if len(args) > 1:
                    try:
                        delay = int(args[1])
                    except:
                        delay = 500
                if delay <= 0:
                    delay = 1
                if floodit['end']:
                    floodit_new_game()
                floodit['timer'] = weechat.hook_timer(delay, 0, 0, 'floodit_timer_cb', '')
        elif args[0] in ('s', 'single'):
            floodit['mode'] = 'single'
            floodit_new_game()
        elif args[0] in ('v', 'versus'):
            floodit['mode'] = 'versus'
            floodit_new_game()
        elif args[0] in ('n', 'new'):
            floodit_new_game()
        elif args[0] in ('q', 'quit'):
            weechat.buffer_close(floodit['buffer'])
        elif args[0] == '+':
            floodit_change_size(+1)
        elif args[0] == '-':
            floodit_change_size(-1)
    return weechat.WEECHAT_RC_OK

def floodit_close_buffer(data, buffer):
    """Called when floodit buffer is closed."""
    global floodit
    if floodit['timer']:
        weechat.unhook(floodit['timer'])
        floodit['timer'] = ''
    floodit['buffer'] = ''
    return weechat.WEECHAT_RC_OK

def floodit_init():
    """Init floodit: create buffer, adjust zoom, new game."""
    global floodit, floodit_settings
    if floodit['buffer']:
        return
    floodit['buffer'] = weechat.buffer_search('python', 'floodit')
    if not floodit['buffer']:
        floodit['buffer'] = weechat.buffer_new('floodit', 'floodit_input_buffer', '', 'floodit_close_buffer', '')
        if floodit['buffer']:
            weechat.buffer_set(floodit['buffer'], 'type', 'free')
            weechat.buffer_set(floodit['buffer'], 'title',
                               'Flood it! | alt-f or mouse: flood, alt-n: new game, alt-+/-: adjust board zoom | '
                               'Command line: (n)ew, (s)ingle, (v)ersus, (d)emo (+delay), +/-: change size, (q)uit')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta2-D', '/floodit left')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta2-C', '/floodit right')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta-f',  '/floodit flood')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta-n',  '/floodit new')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta-+',  '/floodit zoom')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta--',  '/floodit dezoom')
            weechat.buffer_set(floodit['buffer'], 'key_bind_meta-C',  '/floodit computer')
    if floodit['buffer']:
        floodit_adjust_zoom()
        floodit_new_game()

def floodit_flood_xy(board, x, y, color):
    """Flood a board at (x,y) with color."""
    global floodit
    board[y][x] = -1
    if y > 0 and board[y-1][x] == color:
        floodit_flood_xy(board, x, y - 1, color)
    if y < floodit['size'] - 1 and board[y+1][x] == color:
        floodit_flood_xy(board, x, y + 1, color)
    if x > 0 and board[y][x-1] == color:
        floodit_flood_xy(board, x - 1, y, color)
    if x < floodit['size'] - 1 and board[y][x+1] == color:
        floodit_flood_xy(board, x + 1, y, color)

def floodit_flood_end(board, color):
    """End of flood: replace the -1 by color."""
    for y, line in enumerate(board):
        for x, c in enumerate(line):
            if c == -1:
                board[y][x] = color

def floodit_count_color(board, color):
    """Count number of times a color is used in board."""
    global floodit
    count = 0
    for line in board:
        count += line.count(color)
    return count

def floodit_flood(x, y, color):
    """Flood board at (x,y) with color, and check if game has ended."""
    global floodit
    floodit_flood_xy(floodit['board'], x, y, floodit['board'][y][x])
    floodit_flood_end(floodit['board'], color)
    floodit['count'] += 1
    if floodit['mode'] == 'single':
        if floodit_count_color(floodit['board'], floodit['board'][0][0]) == floodit['size'] * floodit['size']:
            floodit['end'] = 'win'
        elif floodit['count'] == floodit['count_max']:
            floodit['end'] = 'lose'
    elif floodit['mode'] == 'versus':
        board = copy.deepcopy(floodit['board'])
        floodit_flood_xy(board, 0, 0, board[0][0])
        count1 = floodit_count_color(board, -1)
        board = copy.deepcopy(floodit['board'])
        floodit_flood_xy(board, floodit['size'] - 1, floodit['size'] - 1, board[floodit['size'] - 1][floodit['size'] - 1])
        count2 = floodit_count_color(board, -1)
        if count1 + count2 == floodit['size'] * floodit['size']:
            if count1 > count2:
                floodit['end'] = 'win'
            elif count1 < count2:
                floodit['end'] = 'lose'
            else:
                floodit['end'] = 'equality'
    floodit_display()

def floodit_build_combs(combs, curlist, maxsize, excludecolor):
    """Build list of combinations to try for computer AI."""
    global floodit
    if len(curlist) >= maxsize:
        combs.append(curlist)
    else:
        curlist.append(-1)
        colors = list(range(0, len(floodit['colors'])))
        random.shuffle(colors)
        for i in colors:
            if i == excludecolor:
                continue
            if i != curlist[-2]:
                curlist[-1] = i
                floodit_build_combs(combs, list(curlist), maxsize, excludecolor)

def floodit_compare_scores(scores1, scores2):
    """Compare two list of scores."""
    sum1 = sum(scores1)
    sum2 = sum(scores2)
    if sum1 > sum2:
        return 1
    elif sum1 < sum2:
        return -1
    else:
        if scores1 > scores2:
            return 1
        elif scores1 < scores2:
            return -1
        else:
            return 0

def floodit_find_best(x, y):
    """Find best color for (x,y) (computer AI)."""
    global floodit
    combs = []
    excludecolor = -1
    if floodit['mode'] == 'versus':
        if x == 0:
            excludecolor = floodit['board'][floodit['size'] - 1][floodit['size'] - 1]
        else:
            excludecolor = floodit['board'][0][0]
    floodit_build_combs(combs, [floodit['board'][y][x]], 3, excludecolor)
    bestscores = []
    bestcolor = 0
    for comb in combs:
        board = copy.deepcopy(floodit['board'])
        scores = []
        for color in comb[1:]:
            floodit_flood_xy(board, x, y, board[y][x])
            floodit_flood_end(board, color)
            floodit_flood_xy(board, x, y, board[y][x])
            scores.append(floodit_count_color(board, -1))
            floodit_flood_end(board, color)
        if floodit_compare_scores(scores, bestscores) > 0:
            bestscores = scores
            bestcolor = comb[1]
    return bestcolor

def floodit_user_flood():
    """Action flood from user, and then computer plays if mode is 'versus'."""
    global floodit
    if floodit['color'] != floodit['board'][0][0]:
        if floodit['mode'] != 'versus' or floodit['color'] != floodit['board'][floodit['size'] - 1][floodit['size'] - 1]:
            floodit_flood(0, 0, floodit['color'])
            if floodit['mode'] == 'versus' and not floodit['end']:
                floodit_flood(floodit['size'] - 1,
                              floodit['size'] - 1,
                              floodit_find_best(floodit['size'] - 1, floodit['size'] - 1))

def floodit_cmd_cb(data, buffer, args):
    """The /floodit command."""
    global floodit
    if args in ('single', 'versus'):
        floodit['mode'] = args
    floodit_init()
    if floodit['buffer']:
        weechat.buffer_set(floodit['buffer'], 'display', '1')
    if not floodit['end']:
        if args == 'left':
            if floodit['color'] > 0:
                floodit['color'] -= 1
            else:
                floodit['color'] = len(floodit['colors']) - 1
            floodit_display()
        elif args == 'right':
            if floodit['color'] < len(floodit['colors']) - 1:
                floodit['color'] += 1
            else:
                floodit['color'] = 0
            floodit_display()
        elif args == 'flood':
            floodit_user_flood()
        elif args == 'computer':
            floodit['color'] = floodit_find_best(0, 0)
            floodit_user_flood()
    if args == 'new':
        floodit_new_game()
    elif args == 'zoom':
        floodit['zoom'] += 1
        floodit_display(True)
    elif args == 'dezoom':
        if floodit['zoom'] > 0:
            floodit['zoom'] -= 1
            floodit_display(True)
    return weechat.WEECHAT_RC_OK

def floodit_mouse_cb(data, hsignal, hashtable):
    """Mouse callback."""
    global floodit
    if not floodit['end']:
        x = int(hashtable.get('_chat_line_x', '-1'))
        y = int(hashtable.get('_chat_line_y', '-1'))
        if x >= 0 and y >= 0:
            color = -1
            if y <= floodit['zoom']:
                multiplier = (floodit['zoom'] + 1) * 4
                add = 2 + ((floodit['zoom'] + 1) * 2)
                for i in range (0, len(floodit['colors'])):
                    if x >= i * multiplier and x < (i * multiplier) + add:
                        color = i
                        break
            elif y >= floodit['zoom'] + 2:
                x = x // ((floodit['zoom'] + 1) * 2)
                y = (y - floodit['zoom'] - 2) // (floodit['zoom'] + 1)
                if y < floodit['size'] and x < floodit['size']:
                    color = floodit['board'][y][x]
            if color >= 0:
                floodit['color'] = color
                floodit_user_flood()
    return weechat.WEECHAT_RC_OK

if __name__ == '__main__' and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, '', ''):
        # set default settings
        version = weechat.info_get("version_number", "") or 0
        for option, value in floodit_settings_default.items():
            if weechat.config_is_set_plugin(option):
                floodit_settings[option] = weechat.config_get_plugin(option)
            else:
                weechat.config_set_plugin(option, value[0])
                floodit_settings[option] = value[0]
            if int(version) >= 0x00030500:
                weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))
        floodit_set_colors()

        # mouse support
        if int(version) >= 0x00030600:
            weechat.key_bind('mouse', floodit_mouse_keys)
            weechat.hook_hsignal('floodit_mouse', 'floodit_mouse_cb', '')

        # detect config changes
        weechat.hook_config('plugins.var.python.%s.*' % SCRIPT_NAME, 'floodit_config_cb', '')

        # add command
        weechat.hook_command(SCRIPT_COMMAND, 'Flood''it game.', '[single|versus]',
                             'single: play in single mode (default)\n'
                             'versus: play versus computer\n\n'
                             'Single mode:\n'
                             '- Choose a color for the upper left square, this will paint '
                             'this square and all squares next to this one (having same color) '
                             'with your color.\n'
                             '- You win if all squares are same color.\n'
                             '- Maximum number of floods is 25, 35 or 50 (according to size).\n\n'
                             'Versus mode:\n'
                             '- You paint the upper left square, WeeChat paints bottom right.\n'
                             '- You can not paint with last color used by WeeChat.\n'
                             '- Game ends when neither you nor WeeChat can paint new squares any more.\n'
                             '- You win if you have more squares of your color than WeeChat.',
                             'single|versus', 'floodit_cmd_cb', '')

        # if buffer already exists (after /upgrade), init floodit
        if weechat.buffer_search('python', 'floodit'):
            floodit_init()



# weejoin
# KittyKatt (kittykatt@teknik.io)
# Released under the BSD license.
# vim: set fileencoding=utf8 ts=4 sw=4 expandtab :

import json
import re
import urllib.parse
import urllib.request
import time

import weechat as w

# Constant used to check if configs are required
REQUIRED = '_required'

w.register('weejoin',
           'KittyKatt (Lefty for weebullet)',
           '1.0',
           'BSD',
           'weejoin pushes notifications from IRC to Join.',
           '', '')

w.hook_print("", "irc_privmsg", "", 1, "priv_msg_cb", "")
w.hook_command(
    "send_push_note",      # command
    "send a push note",    # description
    "[message]"            # arguments description,
    "",                    # argument
    "",
    "",
    "cmd_send_push_note", ""
)
w.hook_command(
    "weejoin",
    "pushes notifications from IRC to Join",
    "[command]",
    "Available commands are:\n"
    "   help        : prints config options and defaults\n"
    "   listdevices : prints a list of all devices associated"
    "                 with your Join API key\n"
    "   listignores : prints a list of channels that highlights "
    "                 won't be pushed for\n"
    "   ignore      : adds a channel to the blacklist\n"
    "   unignore    : removes a channel from the blacklist",
    "",
    "cmd_help", ""
)
configs = {
    "api_key": REQUIRED,
    "away_only": "1",            # only send when away
    "inactive_only": "1",        # only send if buffer inactive
    "device_iden": "all",        # send to all devices
    "ignored_channels": "",      # no ignored channels
    "min_notify_interval": "0",  # seconds, don't notify
                                 #   more often than this
    "debug": "0",                # enable debugging
    "ignore_on_relay": "0",      # if relay connected,
                                 #   don't send push notification
}

last_notification = 0   # 0 seconds from the epoch
# Icon used by Join for notification icon
notification_icon = 'https://upload.wikimedia.org/wikipedia/commons/thumb/a/a3/Weechat_logo.png/64px-Weechat_logo.png'

for option, default_value in configs.items():
    if w.config_get_plugin(option) == "":
        if configs[option] == REQUIRED:
            w.prnt("", w.prefix("error") +
                   "join: Please set option: %s" % option)
            if type(default_value) == "str":
                w.prnt("", "join: /set plugins.var.python.weejoin.%s STRING" % option)
            elif type(default_value) == "int":
                w.prnt("", "join: /set plugins.var.python.weejoin.%s INT" % option)
            else:
                w.prnt("", "join: /set plugins.var.python.weejoin.%s VALUE" % option)
        else:
            w.config_set_plugin(option, configs[option])


def debug(msg):
    if str(w.config_get_plugin("debug")) is not "0":
        w.prnt("", "[weejoin] DEBUG: %s" % str(msg))


def process_devicelist_cb(data, url, status, response, err):
    try:
        devices = json.loads(response)["records"]
        w.prnt("", "Device List:")
        for device in devices:
            if "deviceName" in device:
                w.prnt("", "---\n%s" % device["deviceName"])
            else:
                w.prnt("", "---\nUnnamed")
            w.prnt("", "%s" % device["id"])
    except KeyError:
        w.prnt("", "[weejoin] Error accessing device list: %s" % response)
        return w.WEECHAT_RC_ERROR
    return w.WEECHAT_RC_OK


def get_ignored_channels():
    ignored_channels = w.config_get_plugin("ignored_channels")
    if ignored_channels == "":
        return []
    else:
        return [channel.strip() for channel in ignored_channels.split(',')]


def cmd_help(data, buffer, args):
    # Get current list of ignored channels in list form
    ignored_channels = get_ignored_channels()

    # Used for checking for ignore/unignore commands and getting the arguments
    ignore_command = re.match("^ignore\s+(.+)", args)
    unignore_command = re.match("^unignore\s+(.+)", args)

    if(ignore_command is not None):
        channels_to_ignore = ignore_command.group(1).split(' ')

        for channel in channels_to_ignore:
            if channel not in ignored_channels:
                ignored_channels.append(channel)

        w.config_set_plugin("ignored_channels", ','.join(ignored_channels))
        w.prnt("", "Updated. Ignored channels: %s" % w.config_get_plugin("ignored_channels"))
    elif(unignore_command is not None):
        channels_to_unignore = unignore_command.group(1).split(' ')

        for channel in channels_to_unignore:
            if channel in ignored_channels:
                ignored_channels.remove(channel)

        w.config_set_plugin("ignored_channels", ','.join(ignored_channels))
        w.prnt("", "Updated. Ignored channels: %s" % w.config_get_plugin("ignored_channels"))
    elif(args == "listignores"):
        w.prnt("", "Ignored channels: %s" % w.config_get_plugin("ignored_channels"))
    elif(args == "listdevices"):
        apikey = w.string_eval_expression(w.config_get_plugin("api_key"), {}, {}, {})
        apiurl = "https://joinjoaomgcd.appspot.com/_ah/api/registration/v1/listDevices?apikey=%s" % (apikey)
        w.hook_process("url:" + apiurl, 20000, "process_devicelist_cb", "")
    else:
        w.prnt("", """
WeeJoin requires an API key from your Join account to work. Set your API key with (evaluated):
    /set plugins.var.python.weejoin.api_key <KEY>

WeeJoin will by default only send notifications when you are marked away on IRC. You can change this with:
    /set plugins.var.python.weejoin.away_only [0|1]

WeeJoin will by default send to all devices associated with your Join account. You can change this with:
    /set plugins.var.python.weejoin.device_iden <ID>

WeeJoin can ignore repeated notifications if they arrive too often.  You can set this with (0 or blank to disable):
    /set plugins.var.python.weejoin.min_notify_interval <NUMBER>

You can get a list of your devices from the Join website, or by using
    /weejoin listdevices
""")
    return w.WEECHAT_RC_OK


def process_join_cb(data, url, status, response, err):
    body = None
    headers = {}
    lines = response.rstrip().splitlines()
    status_code = int(lines.pop(0).split()[1])
    for line in lines:
        if body == "":
            body += line
            continue
        header_line = line.split(":", 2)
        if len(header_line) != 2:
            body = ""
            continue
        headers[header_line[0].strip()] = header_line[1].strip()

    # response is the string of http body
    if status == w.WEECHAT_HOOK_PROCESS_ERROR:
        w.prnt("", "[weejoin] Error sending to join: %s - %s" % (status, url))
        return w.WEECHAT_RC_ERROR

    if status_code is 401 or status_code is 403:
        w.prnt("", "[weejoin] Invalid API Token: %s" % (w.string_eval_expression(w.config_get_plugin("api_key"), {}, {}, {})))
        return w.WEECHAT_RC_ERROR
    if status_code is not 200:
        w.prnt("", "[weejoin] Error sending to join: %s - %s - %s" % (url, status_code, body))
        return w.WEECHAT_RC_ERROR

    return w.WEECHAT_RC_OK


def send_push(title, body):
    global last_notification

    interval = w.config_get_plugin("min_notify_interval")
    if interval is not None and interval != "" and int(interval) != 0:
        interval = int(interval)

        earliest_notification = last_notification + int(interval)

        if last_notification is not None and time.time() <= earliest_notification:
            debug("Too soon since last notification, skipping")
            return w.WEECHAT_RC_OK

    last_notification = time.time()

    # check to see if the relay is connected, ignore if so
    check_relays = w.config_string_to_boolean(w.config_get_plugin('ignore_on_relay'))
    CONNECTED_RELAY = False
    if check_relays:
        infolist = w.infolist_get('relay', '', '')
        if infolist:
            while w.infolist_next(infolist):
                status = w.infolist_string(infolist, 'status_string')
                if status == 'connected':
                    CONNECTED_RELAY = True
                    break
            w.infolist_free(infolist)

    if CONNECTED_RELAY is True:
        # we have a relay conected, don't notify
        debug("Relay is connected, not sending push.")
        return w.WEECHAT_RC_OK

    debug("Sending push.  Title: [%s], body: [%s]" % (title, body))

    apikey = w.string_eval_expression(w.config_get_plugin("api_key"), {}, {}, {})
    apiurl = "https://joinjoaomgcd.appspot.com/_ah/api/messaging/v1/sendPush?apikey=%s" % (apikey)
    timeout = 20000  # FIXME - actually use config
    if len(title) is not 0 or len(body) is not 0:
        deviceiden = w.config_get_plugin("device_iden")
        if deviceiden == "all":
            payload = urllib.parse.urlencode({'title': title, 'text': body, 'deviceId': 'group.all', 'icon': notification_icon})
            debug("Full Payload: %s%s" % (apiurl, payload))
        else:
            payload = urllib.parse.urlencode({'title': title, 'text': body, 'deviceId': deviceiden, 'icon': notification_icon})
            debug("Full Payload: %s%s" % (apiurl, payload))
        w.hook_process_hashtable("url:" + apiurl, {"postfields": payload, "header": "1"}, timeout, "process_join_cb", "")


def cmd_send_push_note(data, buffer, args):
    send_push(
        title="Manual Notification from weechat",
        body=args)
    return w.WEECHAT_RC_OK


def priv_msg_cb(data, bufferp, uber_empty,
                tagsn, isdisplayed,
                ishilight, prefix, message):
    """Sends highlighted message to be printed on notification"""

    if w.config_get_plugin("away_only") == "1":
        am_away = w.buffer_get_string(bufferp, 'localvar_away')
    else:
        am_away = True

    if not am_away:
        # TODO: make debug a configurable
        debug("Not away, skipping notification")
        return w.WEECHAT_RC_OK

    # If 'inactive_only' is enabled, we need to check if the notification is
    # coming from the active buffer.
    if w.config_get_plugin("inactive_only") == "1":
        if w.current_buffer() == bufferp:
            # The notification came from the current buffer - don't notify
            debug("Notification came from the active buffer, "
                  "skipping notification")
            return w.WEECHAT_RC_OK

    notif_body = u"<%s> %s" % (prefix, message)

    # Check that it's in a "/q" buffer and that I'm not the one writing the msg
    is_pm = w.buffer_get_string(bufferp, "localvar_type") == "private"
    is_notify_private = re.search(r'(^|,)notify_private(,|$)', tagsn) is not None
    # PM (query)
    if (is_pm and is_notify_private):
        send_push(
            title="Privmsg from %s" % prefix,
            body=notif_body
        )

    # Highlight (your nick is quoted)
    elif (str(ishilight) == "1"):
        bufname = (w.buffer_get_string(bufferp, "short_name") or
                   w.buffer_get_string(bufferp, "name"))

        ignored_channels = get_ignored_channels()

        if bufname not in ignored_channels:
            send_push(
                title="Highlight in %s" % bufname,
                body=notif_body
            )
        else:
            debug("[weejoin] Ignored channel, skipping notification in %s" % bufname)

    return w.WEECHAT_RC_OK



"""
tmux_env.py -- update weechat environment from tmux

Copyright 2013 Aron Griffis <agriffis@n01se.net>
Released under the terms of the GNU General Public License v3

Description:

    This script propagates environment updates from tmux into the weechat
    process, including the script interpreters.  This allows the process
    environment to follow the tmux client, such as the X DISPLAY variable,
    and the DBUS_SESSION_BUS_ADDRESS used for desktop notifications.

    See https://github.com/agriffis/weechat-tmux-env for full usage
    instructions.

History:

    2013-09-30 Aron Griffis <agriffis@n01se.net>
      version 1: initial release

    2014-02-03 Aron Griffis <agriffis@n01se.net>
      version 2: python 2.6 compatible subprocess.check_output()

    2020-01-03 dobbymoodge <john.w.lamb [at] gmail . com> ( https://github.com/dobbymoodge/ )
      version 3: python 3.x compatibility

    2020-10-30 Friedrich Delgado <taupan [at] gmail . com>
      version 3.1: fix python 3.6 compatibility and remove python 2 support
"""

from __future__ import absolute_import, unicode_literals

import weechat as w

import fnmatch
import os
import subprocess

SCRIPT_NAME = "tmux_env"
SCRIPT_AUTHOR = "Aron Griffis <agriffis@n01se.net>"
SCRIPT_VERSION = "3.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Update weechat environment from tmux"

settings = {
    'interval': '30',  # How often in seconds to check for updates

    # For include/exclude, the bare variable name only refers to
    # environment updates, not removals. For removals, include the variable
    # name prefixed by a minus sign. For example, to add/remove exclusively
    # the DISPLAY variable, include="DISPLAY,-DISPLAY"
    #
    # Globs are also accepted, so you can ignore all variable removals with
    # exclude="-*"

    'include': '*,-*',  # Env vars to include, default all
    'exclude': '',      # Env vars to exclude, default all
    }

TIMER = None


def set_timer():
    """Update timer hook with new interval"""

    global TIMER
    if TIMER:
        w.unhook(TIMER)
    TIMER = w.hook_timer(int(w.config_get_plugin('interval')) * 1000,
                         0, 0, 'timer_cb', '')


def config_cb(data, option, value):
    """Reset timer when interval option is updated"""

    if option.endswith('.interval'):
        set_timer()
    return w.WEECHAT_RC_OK


def timer_cb(buffer, args):
    """Check if tmux is attached, update environment"""

    attached = os.access(SOCK, os.X_OK)  # X bit indicates attached
    if attached:
        update_environment()
    return w.WEECHAT_RC_OK


def update_environment():
    """Updates environment from tmux showenv"""

    env = subprocess.check_output(['tmux', 'showenv']).decode()
    for line in env.splitlines():
        name = line.split('=', 1)[0]
        if check_include(name) and not check_exclude(name):
            if name.startswith('-'):
                remove_env(name[1:])
            else:
                add_env(name, line.split('=', 1)[1])


def check_include(name):
    globs = comma_split_config('include')
    return check_match(name, globs)


def check_exclude(name):
    globs = comma_split_config('exclude')
    return check_match(name, globs)


def check_match(name, globs):
    for g in globs:
        if fnmatch.fnmatch(name, g):
            return True


def comma_split_config(name):
    config = w.config_get_plugin(name)
    return filter(None, (s.strip() for s in config.split(',')))


def add_env(name, value):
    old = os.environ.get(name)
    if old != value:
        w.prnt("", "%s: add %s=%r (was %r)" % (SCRIPT_NAME, name, value, old))
        os.environ[name] = value


def remove_env(name):
    old = os.environ.get(name)
    if old is not None:
        w.prnt("", "%s: remove %s (was %r)" % (SCRIPT_NAME, name, old))
        del os.environ[name]


if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
              SCRIPT_DESC, '', ''):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)

    global SOCK
    SOCK = None

    if 'TMUX' in os.environ:
        # We are running under tmux
        socket_data = os.environ['TMUX']
        SOCK = socket_data.rsplit(',', 2)[0]

    if SOCK:
        w.hook_config("plugins.var.python.{}.*".format(SCRIPT_NAME), "config_cb", "")
        set_timer()



#
# Copyright (9) 2024 Jesse McDowell <jessemcd1@gmail.com>
#
# Add IRCCloud avatar image link to WHOIS output
#
# This program is free software: you can redistribute it and/or modify it under the terms of the
# GNU General Public License as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <https://www.gnu.org/licenses/>.
#
# 2024-08-25: Jesse McDowell
#     version 1.0: Initial release


try:
    import weechat
    from weechat import WEECHAT_RC_OK
    import_ok = True
except ImportError:
    print('This script must be run under WeeChat.')
    import_ok = False

import re

def whois311_cb(data, signal, signal_data):
    buffer = weechat.info_get("irc_buffer", signal.split(",")[0])

    userid_text = signal_data.split(" ")[5 if signal_data[0] == "@" else 4]

    userid_match = userid_expression.match(userid_text)
    if userid_match is not None:
        weechat.prnt(buffer, "Avatar image: https://static.irccloud-cdn.com/avatar-redirect/%s" % userid_match.groups()[0])

    return WEECHAT_RC_OK

if __name__ == '__main__' and import_ok:
    weechat.register("irccloud_avatar_link", "Jesse McDowell", "1.0", "GPL3", "Add IRCCloud avatar image link to WHOIS details", "", "")
    userid_expression = re.compile("^[us]id([0-9]+)$")

    weechat.hook_signal("*,irc_in2_311", "whois311_cb", "")



# -*- coding: utf-8 -*-
#
# Script Name: aesthetic.py
# Script Author: Wojciech Siewierski
# Script License: GPL3
# Contact: vifon @ irc.freenode.net

SCRIPT_NAME = 'aesthetic'
SCRIPT_AUTHOR = 'Wojciech Siewierski'
SCRIPT_VERSION = '1.0.6'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC = 'Make messages more A E S T H E T I C A L L Y pleasing.'

import_ok = True

try:
    import weechat
except ImportError:
    print('This script must be run under WeeChat')
    print('You can obtain a copy of WeeChat, for free, at https://weechat.org')
    import_ok = False

weechat_version = 0

import shlex
import sys

def aesthetic_(args):
    for arg in args:
        try:
            arg = arg.decode('utf8')
        except AttributeError:
            pass
        yield " ".join(arg.upper())
        for n, char in enumerate(arg[1:]):
            yield " ".join(" "*(n+1)).join(char.upper()*2)

def aesthetic(args):
    if sys.version_info < (3,):
        return (x.encode('utf8') for x in aesthetic_(args))
    else:
        return aesthetic_(args)

def aesthetic_cb(data, buffer, args):
    for x in aesthetic(shlex.split(args)):
        weechat.command(buffer, x)
    return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        weechat_version = weechat.info_get("version_number", "") or 0
        weechat.hook_command(
            "aesthetic",
            """Format a message like this:

E X A M P L E
X X
A   A
M     M
P       P
L         L
E           E

Each argument is formatted separately, use sh-like quotes for grouping.  For example '/aesthetic foo bar' will send two such blocks while '/aesthetic "foo bar"' would send one larger one.

Use with care to not cause undesirable message spam.""",
            "message", "",
            "",
            "aesthetic_cb", ""
        )



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by Alexander Schremmer <alex@alexanderweb.de>
# Copyright (c) 2013,2020 by Corey Halpin <chalpin@cs.wisc.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# (this script requires WeeChat 0.3.6 or newer)
#
# History:
# 2020-05-10, Sébastien Helleu <flascode@flashtux.org>
#  version 0.7: remove useless list() around .items()
# 2020-04-15, Corey Halpin <chalpin@cs.wisc.edu>
#  version 0.6:
#    * Update for python3, fixing issues pointed out by 2to3.
#    * Fix complaints from pycodestyle.
# 2014-02-15, Corey Halpin <chalpin@cs.wisc.edu>
#  version 0.5:
#    * Improve documentation for the 'server' setting
#    * Change the default for 'server' to bitlbee, as this is probably
#      what most people use.
# 2013-06-24, Priska Herger <policecar@23bit.net>
#  version 0.4: bug fix: if TYPING 0, don't show typing.
# 2013-04-27, Corey Halpin <chalpin@cs.wisc.edu>
#  version 0.3:
#    * Use irc_message_parse to extract nicks
#    * Send typing = 0 at message completion in private buffers
#    * Make server, channel, and timeout configurable w/o editing plugin code.
# 2010-05-20, Alexander Schremmer <alex@alexanderweb.de>
#   version 0.2: also handle users that do not send a TYPING 0 msg before
#     quitting removed InfoList code
# 2010-05-16, Alexander Schremmer <alex@alexanderweb.de>
#   version 0.1: initial release


# Configuration options via /set:
#
# 'channel'
#   description: Name of your bitlbee channel
#   command: /set plugins.var.python.bitlbee_typing_notice.channel &bitlbee
#
# 'server'
#   description: Name of the server running your bitlbee instance.  This will
#     be whatever you type after /connect to get to your bitlbee server
#     (e.g., localhost, bitlbee).
#   command: /set plugins.var.python.bitlbee_typing_notice.server bitlbee
#
# 'timeout'
#   description: Send "not typing" after this many seconds w/o typing.
#   command: /set plugins.var.python.bitlbee_typing_notice.timeout 4
#
# Note, the plugin must be reloaded after either of these settings are changed.
# /python reload works for this.


import weechat as w
import re

SCRIPT_NAME = "bitlbee_typing_notice"
SCRIPT_AUTHOR = "Alexander Schremmer <alex@alexanderweb.de>"
SCRIPT_VERSION = "0.7"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Sends and displays bitlbee typing notices."


# Record nicks who sent typing notices.
#   key = subject nick, val = typing level.
typing = {}


# Nicks to whom we're sending typing notices.
#  key = target nick, val = sequence number used to determine when the typing
#    notice should be removed.
sending_typing = {}


def channel_has_nick(server, channel, nick):
    buffer = w.buffer_search("", "%s.%s" % (server, channel))
    return bool(w.nicklist_search_nick(buffer, "", nick))


# Callback which checks for ctcp typing notices sent to us.
# Updates typing data, hides the ctcp notices.
def ctcp_cb(data, modifier, modifier_data, string):
    if modifier_data != w.config_get_plugin("server"):
        return string
    msg_hash = w.info_get_hashtable(
        "irc_message_parse", {"message": string})
    if msg_hash["command"] != "PRIVMSG":
        return string
    match = re.search('\001TYPING ([0-9])\001', msg_hash["arguments"])
    if match:
        nick = msg_hash["nick"]
        typing_level = int(match.groups()[0])
        if typing_level == 0 and nick in typing:
            del typing[nick]
        elif typing_level > 0:
            typing[nick] = typing_level
        w.bar_item_update("bitlbee_typing_notice")
        return ""
    else:
        return string


def stop_typing(data, signal, signal_data):
    msg_hash = w.info_get_hashtable(
        "irc_message_parse", {"message": signal_data})
    if msg_hash["nick"] in typing:
        del typing[msg_hash["nick"]]
    w.bar_item_update("bitlbee_typing_notice")
    return w.WEECHAT_RC_OK


def typed_char(data, signal, signal_data):
    buffer = w.current_buffer()
    input_s = w.buffer_get_string(buffer, 'input')
    server = w.buffer_get_string(buffer, 'localvar_server')
    channel = w.buffer_get_string(buffer, 'localvar_channel')
    buffer_type = w.buffer_get_string(buffer, 'localvar_type')

    if server != w.config_get_plugin("server") or input_s.startswith("/"):
        return w.WEECHAT_RC_OK
    if buffer_type == "private":
        if len(input_s) == 0:
            send_typing(channel, 0)  # Line sent or deleted -- no longer typing
        else:
            send_typing(channel, 1)
    elif channel == w.config_get_plugin("channel"):
        nick_completer = w.config_string("weechat.completion.nick_completer")
        parts = input_s.split(":", 1)
        if len(parts) > 1:
            nick = parts[0]
            send_typing(nick, 1)
    return w.WEECHAT_RC_OK


def typing_disable_timer(data, remaining_calls):
    nick, cookie = data.rsplit(":", 1)
    cookie = int(cookie)
    if nick in sending_typing and sending_typing[nick] == cookie:
        send_typing_ctcp(nick, 0)
        del sending_typing[nick]
    return w.WEECHAT_RC_OK


def send_typing_ctcp(nick, level):
    if not channel_has_nick(w.config_get_plugin("server"),
                            w.config_get_plugin("channel"), nick):
        return

    server = w.config_get_plugin("server")
    channel = w.config_get_plugin("channel")
    buffer = w.buffer_search(
      "irc", "%s.%s" % (server, channel))
    w.command(buffer, "/mute -all /ctcp %s TYPING %i" % (nick, level))


def send_typing(nick, level):
    if level == 0 and nick in sending_typing:
        send_typing_ctcp(nick, 0)
        del sending_typing[nick]
    elif level > 0:
        if nick not in sending_typing:
            send_typing_ctcp(nick, level)
        cookie = sending_typing.get(nick, 0) + 1
        sending_typing[nick] = cookie
        timeout = int(1000 * float(w.config_get_plugin('timeout')))
        w.hook_timer(
          timeout, 0, 1, "typing_disable_timer", "%s:%i" % (nick, cookie))


def typing_notice_item_cb(data, buffer, args):
    if typing:
        msgs = []
        for key, value in typing.items():
            msg = key
            if value == 2:
                msg += " (stale)"
            msgs.append(msg)
        return "typing: " + ", ".join(sorted(msgs))
    return ""


# Main
if __name__ == "__main__":
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                  SCRIPT_DESC, "", ""):

        if not w.config_get_plugin('channel'):
            w.config_set_plugin('channel', "&bitlbee")
        if not w.config_get_plugin('server'):
            w.config_set_plugin('server', "bitlbee")
        if not w.config_get_plugin('timeout'):
            w.config_set_plugin('timeout', "4")

        server = w.config_get_plugin("server")

        w.hook_signal("input_text_changed", "typed_char", "")
        w.hook_signal(server+",irc_in_quit", "stop_typing", "")
        w.hook_signal(server+",irc_in_privmsg", "stop_typing", "")
        w.bar_item_new('bitlbee_typing_notice', 'typing_notice_item_cb', '')
        w.hook_modifier("irc_in_privmsg", "ctcp_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 by Ricky Brent <ricky@rickybrent.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
"""Automatically merge new irc buffers according to defined rules.

History:
    * 2017-03-22, Ricky Brent <ricky@rickybrent.com>:
          version 0.1: initial release
    * 2018-03-02, Brady Trainor <mail@bradyt.com>:
          version 0.2: fix the merge command
"""

from __future__ import print_function
import re
try:
    import weechat
    IMPORT_OK = True
except ImportError:
    print('Script must be run under weechat. http://www.weechat.org')
    IMPORT_OK = False

VERSION = '0.2'
NAME = 'automerge'
AUTHOR = 'Ricky Brent <ricky@rickybrent.com>'
DESC = 'Merge new irc buffers according to defined rules.'

DELIMITER1 = '|@|'
DELIMITER2 = '|!|'

CMD_DESC = '''List, add, delete or apply automerge rules.

Adding a rule takes two parameters: a regular expression to match the target, and \
a regular expression, integer, or the special string 'server' to match the \
destination.

Optionally, the first parameter can be omitted; in this case, the active buffer name will be used.

Rules can be deleted by their regular expression or their index.'''
CMD_LIST = ['list', 'add', 'delete', 'bufferlist', 'apply']
CMD_COMPLETE = '||'.join(CMD_LIST)

def find_merge_id(buf, merge):
    """Find the id of the buffer to merge to."""
    mid = -1
    if merge.isdigit():
        mid = merge
    elif merge == "server":
        server = weechat.buffer_get_string(buf, 'localvar_server')
        infolist = weechat.infolist_get("buffer", "", "")
        while weechat.infolist_next(infolist) and mid < 0:
            if weechat.infolist_string(infolist, "plugin_name") == "irc":
                buf2 = weechat.infolist_pointer(infolist, "pointer")
                server2 = weechat.buffer_get_string(buf2, 'localvar_server')
                if server == server2:
                    mid = weechat.infolist_integer(infolist, 'number')
        weechat.infolist_free(infolist)
    else:
        infolist = weechat.infolist_get("buffer", "", "")
        prog = re.compile(merge)
        while weechat.infolist_next(infolist) and mid < 0:
            if prog.match(weechat.infolist_string(infolist, "full_name")):
                mid = weechat.infolist_integer(infolist, 'number')
        weechat.infolist_free(infolist)
    return mid

def get_rules():
    """Return a list of rules."""
    rules = weechat.config_get_plugin('rules')
    if rules:
        return rules.split(DELIMITER1)
    else:
        return []

def cb_signal_apply_rules(data, signal, buf):
    """Callback for signal applying rules to the buffer."""
    name = weechat.buffer_get_string(buf, "full_name")
    rules = get_rules()
    for rule in rules:
        pattern, merge = rule.split(DELIMITER2)
        if re.match(pattern, name):
            mid = find_merge_id(buf, merge)
            if mid >= 0:
                weechat.command(buf, "/buffer merge " + str(mid))
    return weechat.WEECHAT_RC_OK

def cb_command(data, buf, args):
    """Handle user commands; add/remove/list rules."""
    list_args = args.split(" ")
    commands = {
        'list': cb_command_list,
        'bufferlist': cb_command_bufferlist,
        'add': cb_command_add,
        'delete': cb_command_delete,
        'del': cb_command_delete,
        'apply': cb_command_apply
    }
    if len(list_args) == 0:
        weechat.command(buf, '/help ' + NAME)
        return weechat.WEECHAT_RC_OK
    elif list_args[0] in commands:
        commands[list_args[0]](data, buf, list_args)
    else:
        weechat.prnt(buf, ("[" + NAME + "] Bad option for /" + NAME + " "
                           "command, try '/help " + NAME + "' for more info."))
    return weechat.WEECHAT_RC_OK

def cb_command_list(data, buf, list_args):
    """Print a list all rules."""
    weechat.prnt('', "[" + NAME + "] rules (list)")
    rules = get_rules()
    if len(rules) == 0:
        return weechat.WEECHAT_RC_OK
    for idx, rule in enumerate(rules):
        pattern, merge = rule.split(DELIMITER2)
        weechat.prnt('', '  ' + str(idx) + ": " + pattern + ' = ' + merge)
    return weechat.WEECHAT_RC_OK

def cb_command_bufferlist(data, buf, list_args):
    """Print a list of all buffer names."""
    infolist = weechat.infolist_get("buffer", "", "")
    weechat.prnt('', "[" + NAME + "] buffer list")
    while weechat.infolist_next(infolist):
        weechat.prnt('', '  ' + weechat.infolist_string(infolist, "full_name"))
    weechat.infolist_free(infolist)
    return weechat.WEECHAT_RC_OK

def cb_command_add(data, buf, list_args):
    """Add a rule."""
    rules = get_rules()
    if len(list_args) == 3:
        rule = list_args[1]
        match = list_args[2]
    elif len(list_args) == 2:
        rule = weechat.buffer_get_string(buf, "name")
        match = list_args[1]
    else:
        return bad_command(buf)
    rules.append(DELIMITER2.join([rule, match]))
    weechat.config_set_plugin('rules', DELIMITER1.join(rules))
    weechat.prnt('', "[" + NAME + "] rule added: " + rule + " => " + match)
    return weechat.WEECHAT_RC_OK

def cb_command_delete(data, buf, list_args):
    """Delete a rule."""
    rules = get_rules()
    if len(list_args) == 2:
        rules2 = []
        for idx, rule in enumerate(rules):
            pattern, dummy = rule.split(DELIMITER2)
            if str(idx) != list_args[1] and pattern != list_args[1]:
                rules2.append(rule)
        weechat.config_set_plugin('rules', DELIMITER1.join(rules2))
        if len(rules2) == len(rules):
            weechat.prnt('', "[" + NAME + "] rule not found")
        else:
            weechat.prnt('', "[" + NAME + "] rule deleted")
    return weechat.WEECHAT_RC_OK

def cb_command_apply(data, buf, list_args):
    """Apply the rules the all existing buffers; useful when testing a new rule."""
    infolist = weechat.infolist_get("buffer", "", "")
    while weechat.infolist_next(infolist):
        buf2 = weechat.infolist_pointer(infolist, "pointer")
        cb_signal_apply_rules(data, None, buf2)
    weechat.infolist_free(infolist)
    return weechat.WEECHAT_RC_OK

def bad_command(buf):
    """Print an error message about the command."""
    weechat.prnt(buf, ("[" + NAME + "] Bad option for /" + NAME + " "
                       "command, try '/help " + NAME + "' for more info."))
    return weechat.WEECHAT_RC_OK

if IMPORT_OK:
    weechat.register(NAME, AUTHOR, VERSION, 'GPL2', DESC, '', '')
    weechat.hook_signal('irc_channel_opened', 'cb_signal_apply_rules', '')
    weechat.hook_signal('irc_pv_opened', 'cb_signal_apply_rules', '')
    weechat.config_set_desc_plugin('rules', 'Rules to follow when automerging.')
    if not weechat.config_is_set_plugin('rules'):
        weechat.config_set_plugin('rules', '')
    weechat.hook_command(NAME, CMD_DESC, '[' + '|'.join(CMD_LIST) + ']',
                         '', CMD_COMPLETE, 'cb_command', '')
